from .....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func, \
    instance_he_by_module_name, set_self_x_item_value_by_setter_with_agr
from ....types.he import He


class Node(He):
    def __init__(self, u):
        super().__init__(u)
        if type(self) is Node: raise TypeError("Node 不能直接实例化，只能由子类调用。")
        from ..nodes.wrap import Wrap
        from .. import Htmdoc
        if isinstance(u, Htmdoc):
            super().__init__(u)
            self.__x__['document'] = u
        elif isinstance(u, Wrap):
            super().__init__(u.document)
            self.__x__['document'] = u.document
            self.__x__['senior'] = u
            self.senior.juniors[self.__id__] = self
        else:
            raise TypeError(f'Node __init__ 参数 u {type(u)} 只能是 Htmldoc 或者 Wrap。')
        self.document.elements[self.__id__] = self

    def ___senior___(self, senior=None):
        if senior is None:
            if self.has_senior():
                del self.senior.juniors[self.__id__]
                del self.__x__['senior']
                return True
            return False
        from..nodes.wrap import Wrap
        if not isinstance(self.senior, Wrap): raise TypeError(f'Node 的父节点只能是 Wrap。')
        if self.has_senior(): self.___senior___()
        senior.juniors[self.__id__] = self
        self.__x__['senior'] = senior
        return True

    @property
    @parse_self_x_item_value()
    def document(self): return
    @property
    @parse_self_x_item_value()
    def senior(self): return
    def has_senior(self): return self.__x__.get('senior', None) is not None

    @staticmethod
    def is_comment(): return False
    @staticmethod
    def is_script(): return False
    @staticmethod
    def is_tag(): return False
    @staticmethod
    def is_text(): return False
    @staticmethod
    def is_wrap(): return False

    @property
    @parse_self_x_item_value()
    def ___anchored_to___(self): return
    def ___anchor_to___(self, node): self.__x__['___anchored_to___'] = node
    def ___is_anchored_to___(self): return self.__x__.get('___anchored_to___', None) is not None
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___anchors___(self): return {}
    def ___anchor___(self, sn:int):
        if sn == 0: raise RuntimeError('锚点位置不支持参数 sn = 0')
        if sn not in self.___anchors___:
            self.___anchors___[sn] = self.document.wrap()
            self.___anchors___[sn].___anchor_to___(self)
        return self.___anchors___[sn].___anchor_to___(self)
    def before(self, sn:int): return self.___anchor___(sn or -sn)
    @property
    def before1(self): return self.___anchor___(1)
    @property
    def before2(self): return self.___anchor___(2)
    @property
    def before3(self): return self.___anchor___(3)
    @property
    def before4(self): return self.___anchor___(4)
    @property
    def before5(self): return self.___anchor___(5)
    def after(self, sn:int): return self.___anchor___(-sn or sn)
    @property
    def after1(self): return self.___anchor___(-1)
    @property
    def after2(self): return self.___anchor___(-2)
    @property
    def after3(self): return self.___anchor___(-3)
    @property
    def after4(self): return self.___anchor___(-4)
    @property
    def after5(self): return self.___anchor___(-5)
    ####################################################################################################
    # Content 专用
    @property
    @parse_self_x_item_value()
    def ___content___(self): return
    @___content___.setter
    @set_self_x_item_value_by_setter_with_agr()
    def ___content___(self, content: str): return
    ####################################################################################################
    # Wrap 专用
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___juniors___(self): return {}

    @instance_he_by_module_name(module_name='.nodes#', module_name_striped=True)
    def ___comment___(self, content: str = None): return
    @instance_he_by_module_name(module_name='.nodes#', module_name_striped=True)
    def ___script___(self, content: str = None): return
    @instance_he_by_module_name(module_name='.nodes#', module_name_striped=True)
    def ___text___(self, content: str = None): return
    @instance_he_by_module_name(module_name='.nodes#', module_name_striped=True)
    def ___wrap___(self, content=None): return

    def ___append___(self, element=None):
        if isinstance(element, list):
            return [self.___append___(e) for e in element]
        if isinstance(element, str|float|int):
            return self.___text___(element)
        if isinstance(element, Node):
            element.___senior___(self)
            return element
        return None
    ####################################################################################################
    # Tag 专用
    @property
    @parse_self_x_item_value()
    def ___name___(self): return
    @___name___.setter
    @set_self_x_item_value_by_setter_with_agr()
    def ___name___(self, name: str): return
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___attributes___(self): return {}

    @property
    @parse_self_x_item_value()
    def ___inner_anchored_to___(self): return
    def ___inner_anchor_to___(self, node): self.__x__['___inner_anchored_to___'] = node
    def ___is_inner_anchored_to___(self): return self.__x__.get('___inner_anchored_to___', None) is not None
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___inner_anchors___(self): return {}
    def ___inner_anchor___(self, sn:int):
        if sn == 0: raise RuntimeError('锚点位置不支持参数 sn = 0')
        if sn not in self.___inner_anchors___:
            self.___inner_anchors___[sn] = self.document.wrap()
            self.___inner_anchors___[sn].___inner_anchor_to___(self)
        return self.___inner_anchors___[sn].___inner_anchor_to___(self)
    def ___inner_before___(self, sn:int): return self.___inner_anchor___(sn)
    @property
    def ___inner_before1___(self): return self.___inner_anchor___(1)
    @property
    def ___inner_before2___(self): return self.___inner_anchor___(2)
    @property
    def ___inner_before3___(self): return self.___inner_anchor___(3)
    @property
    def ___inner_before4___(self): return self.___inner_anchor___(4)
    @property
    def ___inner_before5___(self): return self.___inner_anchor___(5)
    def ___inner_after___(self, sn:int): return self.___inner_anchor___(sn)
    @property
    def ___inner_after1___(self): return self.___inner_anchor___(-1)
    @property
    def ___inner_after2___(self): return self.___inner_anchor___(-2)
    @property
    def ___inner_after3___(self): return self.___inner_anchor___(-3)
    @property
    def ___inner_after4___(self): return self.___inner_anchor___(-4)
    @property
    def ___inner_after5___(self): return self.___inner_anchor___(-5)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___output_pairable___(self): return True
    @___output_pairable___.setter
    @set_self_x_item_value_by_setter_with_agr()
    def ___output_pairable___(self, pairable: bool): return
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___output_brealable___(self): return True
    @___output_brealable___.setter
    @set_self_x_item_value_by_setter_with_agr()
    def ___output_brealable___(self, brealable: bool): return
    ####################################################################################################

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___level___(self):
        if self.is_wrap():
            if self.___is_anchored_to___():
                return self.___anchored_to___.___level___
            if self.___is_inner_anchored_to___():
                return self.___inner_anchored_to___.___level___ + 1
        if self.has_senior():
            return self.senior.___output_level___ + 1
        return 0

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___output_level___(self):
        if self.is_wrap():
            if self.___is_anchored_to___():
                return self.___anchored_to___.___output_level___
            if self.___is_inner_anchored_to___():
                return self.___inner_anchored_to___.___output_level___ + 1
        if self.has_senior():
            if self.is_wrap():
                return self.senior.___output_level___
            return self.senior.___output_level___ + 1
        return 0

    def __str__(self):
        rs = []
        # 输出前置锚点
        rs.extend(str(self.___anchors___[k]) for k in sorted((k for k in self.___anchors___ if k > 0), reverse=True))
        rs.append(f'{self.___level___ * '\t'}<{self.__class__.__name__}'
                  f' level={self.___level___}'
                  f' id={self.__id__}'
                  f' senior="{f'{type(s:=self.senior if self.has_senior() else self.document).__name__}@{id(s)}'}"'
                  f' juniors="{','.join([f'{type(i).__name__}@{id(i)}' for i in self.___juniors___.values()])}"'
                  f' class="{'.'.join([c.__name__ for c in self.__class__.__mro__ if issubclass(c, Node)].__reversed__())}"'
                  f' output_level={self.___output_level___}'
                  f'>'
                  )
        if self.is_wrap() or self.is_tag():
            # 输出内部前置锚点
            if self.is_tag(): rs.extend(i.__str__() for k, i in sorted(((k, v) for k, v in self.___inner_anchors___.items() if k > 0), key=lambda x: x[0], reverse=True))
            for i in self.___juniors___.values(): rs.append(i.__str__())
            # 输出内部前置锚点
            if self.is_tag(): rs.extend(i.__str__() for k, i in sorted(((k, v) for k, v in self.___inner_anchors___.items() if k > 0), key=lambda x: x[0], reverse=True))
        # 输出后置锚点
        rs.extend(str(self.___anchors___[k]) for k in sorted((k for k in self.___anchors___ if k < 0), reverse=True))
        return '\n'.join(r for r in rs if r)

    def output(self):
        mini = self.document.configs.output_minimized
        rs = []
        # 输出前置锚点
        rs.extend(self.___anchors___[k].output() for k in sorted((k for k in self.___anchors___ if k > 0), reverse=True))
        if self.is_comment():
            if not self.document.configs.output_minimized and self.document.configs.output_comments:
                self.document.___output_next_break___ = True
                rs.extend((
                    '\n' + self.document.configs.output_indentation * self.___output_level___,
                    f'<!--[{self.___content___.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')}]-->'
                ))
        if self.is_script():
            self.document.___output_next_break___ = False
            rs.append(self.___content___)
        if self.is_tag():
            remove_space = not mini and self.___output_level___ > 0 and self.document.configs.output_unspaced
            if not mini:
                if self.document.___output_next_break___:
                    if remove_space: rs.append('<!--')
                    rs.append('\n' + self.document.configs.output_indentation * self.___output_level___)
                    if remove_space: rs.append('-->')
            ##################################################
            # Attributes:
            rs.append(f'<{self.___name___}')
            for k, v in self.___attributes___.items():
                if v is None: rs.append(f' {k}')
                else: rs.append(f' {k}="{v}"')
            if not self.___output_pairable___:
                rs.append(' />')
            else:
                rs.append('>')
                self.document.___output_next_break___ = True
                ##################################################
                # 输出内部前置锚点
                rs.extend(self.___inner_anchors___[k].output() for k in sorted((k for k in self.___inner_anchors___ if k > 0), reverse=True))
                ##################################################
                # Inner Main
                ris = [r for i in self.___juniors___.values() if (r:=i.output())]
                rs.extend(ris)
                if not mini:
                    if self.document.___output_next_break___ and ris:
                        rs.append("\n" + self.document.configs.output_indentation * self.___output_level___)
                rs.append('</' + self.___name___ + '>')
                self.document.___output_next_break___ = self.___output_brealable___
                ##################################################
                # 输出内部后置锚点
                rs.extend(self.___inner_anchors___[k].output() for k in sorted((k for k in self.___inner_anchors___ if k < 0), reverse=True))
        if self.is_text():
            self.document.___output_next_break___ = False
            rs.append(self.___content___.replace('<', '&lt;').replace('>', '&gt;'))
        if self.is_wrap():
            rs.extend(i.output() for i in self.___juniors___.values())
        # 输出后置锚点
        rs.extend(self.___anchors___[k].output() for k in sorted((k for k in self.___anchors___ if k < 0), reverse=True))
        return ''.join(rs)


from ....types.hd import Hd


class Defaults(Hd):
    def __getitem__(self, name: str, default=None):
        if name in self.__u__.___hooks___: return self.__u__.___hooks___[name][0].___defaults___.__getitem__(name)
        if name in self: return super().__getitem__(name)
        return None


from ...__data__ import Data
from .....queries.plugins.conditions import Conditions


class Update(Data):
    @property
    def conditions(self) -> Conditions: ...

    def execute(self, data: dict) -> None: ...




from ......functions.decorators import parse_self_x_item_value_or_set_by_func
from ...nodes.tag import Tag


class Html(Tag):
    def __init__(self, u, attributes:dict[str, str|int|float|None]=None):
        Tag.__init__(self, u, None, attributes)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def head(self): return self.inner_before1.Head()
    @property
    @parse_self_x_item_value_or_set_by_func()
    def body(self): return self.Body()



from .__tags_creater__ import TagsCreater
from .configs import Configs
from .node import Node
from .nodes.comment import Comment
from .nodes.script import Script
from .nodes.tag import Tag
from .nodes.text import Text
from .nodes.wrap import Wrap
from .tags.html import Html
from .tags.link import Link
from ...types.he import He


class Htmdoc(He, TagsCreater):
    def __init__(self, u: He, configs:dict=None): ...
    @property
    def elements(self)-> dict[int, Node]: ...
    @property
    def configs(self)-> Configs: ...
    @property
    def root(self)-> Html: ...

    def comment(self, content: str = None)-> Comment: ...
    def script(self, content: str = None)-> Script: ...
    def text(self, content: str = None)-> Text: ...
    def wrap(self, content=None)-> Wrap: ...


    @staticmethod
    def ___auto_create_tag___(u, name:str, args:tuple|list, kwargs:dict)-> Tag: ...

    @property
    def ___output_next_break___(self)-> bool: ...
    @___output_next_break___.setter
    def ___output_next_break___(self, value)-> bool: ...

    def output(self)-> str: ...


    @property
    def ___css_links___(self)-> list[str]: ...
    @property
    def ___js_links___(self)-> list[str]: ...
    @property
    def ___css_scripts___(self)-> list[str]: ...
    @property
    def ___js_scripts___(self)-> list[str]: ...
    def add_css_link(self, url)-> None: ...
    def add_js_link(self, url)-> None: ...
    def add_css_code(self, code)-> None: ...
    def add_js_code(self, code)-> None: ...

    """def add_library_css_link(self, path)-> None: ...
    def add_library_js_link(self, path)-> None: ...

    def library_file_url(self, *names)-> str: ...
    def library_file(self, *names)-> str: ..."""

    def add_css_link_node(self, url: str)-> Link: ...
    def add_js_link_node(self, url: str)-> Script: ...
    def add_library_css_link_node(self, *names)-> str: ...
    def add_library_js_link_node(self, *names)-> str: ...
    def add_css_script_node(self, url: str)-> Link: ...
    def add_js_script_node(self, url: str)-> Script: ...



from .menus import Menus


class Dropdown(dict):
    def __init__(self, title=None):
        dict.__init__(self)
        self.update({
            'title': title,
            'menus': Menus(self),
        })

    @property
    def title(self): return self.get('title')
    @title.setter
    def title(self, value): self['title'] = value

    @property
    def menus(self): return self.get('menus')

    def is_menu(self): return False
    def is_dropdown(self): return True


from ..address import Address
from .... import WebSystem
from ......types.hd import Hd
from ....configs import Configs
from ....clients import Clients
from .. import Client
from .headers import Headers
from .arguments import Arguments
from .form import Form


class Request(Hd):
    @property
    def __websystem__(self)-> WebSystem: ...
    @property
    def __clients__(self)-> Clients: ...
    @property
    def __client__(self)-> Client: ...
    @property
    def __configs__(self)-> Configs: ...
    @property
    def address(self)-> Address: ...
    @property
    def headers(self)-> Headers: ...
    @property
    def arguments(self)-> Arguments: ...
    @property
    def form(self)-> Form: ...
    @property
    def method(self)-> str: ...
    def is_method_post(self)-> bool: ...
    def is_method_get(self)-> bool: ...
    @property
    def url(self)-> str: ...
    @property
    def path(self)-> str: ...
    @property
    def remote_addr(self)-> str: ...



from ...node import Node


class Text(Node):
    def __init__(self, u, content:str=None):
        super().__init__(u)
        self.___content___ = content or ''

    @property
    def content(self): return self.___content___

    @staticmethod
    def is_text(): return True


from ......functions.decorators import parse_self_x_item_value_or_set_by_instance_he_by_module_name
from ....htmdoc import Htmdoc as PluginHtmDoc


class Htmdoc(PluginHtmDoc):
    def __init__(self, website):
        super().__init__(website)
        self.configs.hook('language', self.__websystem__.configs)
        self.configs.hook('output_minimized', self.__websystem__.configs, 'html_output_minimized')
        self.configs.hook('output_unspaced', self.__websystem__.configs, 'html_output_unspaced')
        self.configs.hook('output_comments', self.__websystem__.configs, 'html_output_comments')
        self.configs.hook('output_indentation', self.__websystem__.configs, 'html_output_indentation')
        self.configs.hook('title_delimiter', self.__websystem__.configs, 'html_title_delimiter')
        self.configs.hook('title_ascending', self.__websystem__.configs, 'html_title_ascending')

    @property
    def __websystem__(self): return self.__website__.websystem
    @property
    def __website__(self): return self.__u__
    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def areas(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def caches(self): return



from ...nodes.tag import Tag


class Img(Tag):
    __hd_tag_pairable__ = False
    def __init__(self, u, url=None, width=None, height=None, title:str= None, attributes:dict[str, str|int|float|None]=None):
        super().__init__(u, "img", attributes)
        if url: self.attributes['src'] = url
        if width: self.attributes['width'] = width
        if height: self.attributes['height'] = height
        if title: self.attributes['alt'] = title




from ..cls_type import ClsType
from ..configs import Configs
from ..folder import Folder
from ...types.he import He
from ...types.hd import Hd
from ..multilinguals.package import Package


class Multilinguals(Hd):
    def __init__(self, u: He, end_class: type = None, start_class: type = None, folder: Folder | str = None, hooked_configs: Configs = None, hooked_config_name: str = None): ...

    @property
    def ___tree_cls_types___(self)-> list[ClsType]: ...

    @property
    def ___hooked_configs___(self)-> Configs | None: ...
    @property
    def ___hooked_config_name___(self)-> str | None: ...

    @property
    def folder(self)-> Folder: ...

    def package(self, language: str)-> Package: ...

    @property
    def default(self)-> Package: ...

    @property
    def runtime(self)-> Package: ...

    def ml(self, code: str, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_name(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_abbreviation(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_description(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_link_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_page_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...
    def ml_breadcrumb_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str: ...



from ... import Htmdoc
from ...nodes.tag import Tag
from ...nodes.wrap import Wrap


class Body(Tag):
    def __init__(self, u: Htmdoc | Wrap, content=None, attributes:dict[str, str|int|float|None]=None): ...



from .....functions.decorators import parse_self_x_item_value_or_set_by_func
from ....types.he import He


class Connection(He):

    def is_connected(self, recheck=False): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def queried_sqls(self): return []
    @property
    def last_queried_sql(self): return self.queried_sqls[-1] if len(self.queried_sqls) > 0 else ''
    @property
    @parse_self_x_item_value_or_set_by_func()
    def queried_times(self): return len(self.queried_sqls)

    def query(self, sql): return

    # noinspection PyMethodMayBeStatic
    def search_tables(self, keyword=None, startwith=None, endwith=None): raise RuntimeError('函数必须在继承类中重写')

    def is_table_exists(self, table): return table in self.search_tables()

    def create_table(self, table): raise RuntimeError('函数必须在继承类中重写')
    def delete_table(self, table): raise RuntimeError('函数必须在继承类中重写')

    def search_table_column_structures(self, table, keyword=None, startwith=None, endwith=None): raise RuntimeError('函数必须在继承类中重写')
    def search_table_columns(self, table, keyword=None, startwith=None, endwith=None): raise RuntimeError('函数必须在继承类中重写')

    def create_table_column(self, table, column, datatype=None, length=0, unique=False, nullable=False): raise RuntimeError('函数必须在继承类中重写') 
    def set_table_column_quniue(self, table, column): raise RuntimeError('函数必须在继承类中重写')

    def search_table_data_multiple(self, sql): raise RuntimeError('函数必须在继承类中重写')



from .....types.he import He
from ... import Database
from .. import Tables
from .columns import Columns
from .data import Data
from ...configs import Configs
from ...connection import Connection


class Table(He):
    def __init__(self, u: Tables, name: str): ...
    @property
    def __database__(self)-> Database: ...
    @property
    def __tables__(self)-> Tables: ...
    @property
    def __configs__(self)-> Configs: ...
    @property
    def __connection__(self)-> Connection: ...
    @property
    def name(self)-> str: ...
    @property
    def columns(self)-> Columns: ...
    @property
    def data(self)-> Data: ...
    @property
    def fullname(self)-> str: ...

    def exists(self)-> bool: ...
    def create(self)-> bool: ...
    def delete(self)-> bool: ...



from .tags.a import A
from .tags.b import B
from .tags.base import Base
from .tags.body import Body
from .tags.button import Button
from .tags.code import Code
from .tags.dd import Dd
from .tags.div import Div
from .tags.dl import Dl
from .tags.dt import Dt
from .tags.form import Form
from .tags.h1 import H1
from .tags.h2 import H2
from .tags.h3 import H3
from .tags.h4 import H4
from .tags.h5 import H5
from .tags.h6 import H6
from .tags.head import Head
from .tags.hr import Hr
from .tags.html import Html
from .tags.i import I
from .tags.img import Img
from .tags.input import Input
from .tags.input_text import InputText
from .tags.li import Li
from .tags.link import Link
from .tags.meta import Meta
from .tags.nav import Nav
from .tags.pre import Pre
from .tags.script import Script
from .tags.span import Span
from .tags.style import Style
from .tags.table import Table
from .tags.tbody import Tbody
from .tags.td import  Td
from .tags.th import  Th
from .tags.thead import Thead
from .tags.title import Title
from .tags.tr import Tr
from .tags.ul import Ul

class TagsCreater:
    # a
    def A(self, title=None, url: str = None, attributes:dict[str, str|int|float|None]=None) -> A: ...
    # b
    def B(self, content=None, attributes:dict[str, str|int|float|None]=None) -> B:...
    # base
    def Base(self, attributes:dict[str, str|int|float|None]=None) -> Base:...
    # body
    def Body(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Body:...
    # button
    def Button(self, title=None, attributes:dict[str, str|int|float|None]=None) -> Button:...
    # code
    def Code(self, content:str=None, attributes:dict[str, str|int|float|None]=None) -> Code:...
    # dd
    def Dd(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Dd:...
    # div
    def Div(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Div:...
    # dl
    def Dl(self, attributes:dict[str, str|int|float|None]=None) -> Dl:...
    # dt
    def Dt(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Dt:...
    # form
    def Form(self, action: str = None, attributes:dict[str, str|int|float|None]=None) -> Form:...
    # h1
    def H1(self, content=None, attributes:dict[str, str|int|float|None]=None) -> H1:...
    # h2
    def H2(self, content=None, attributes:dict[str, str|int|float|None]=None) -> H2:...
    # h3
    def H3(self, content=None, attributes:dict[str, str|int|float|None]=None) -> H3:...
    # h4
    def H4(self, content=None, attributes:dict[str, str|int|float|None]=None) -> H4:...
    # h5
    def H5(self, content=None, attributes:dict[str, str|int|float|None]=None) -> H5:...
    # h6
    def H6(self, content=None, attributes:dict[str, str|int|float|None]=None) -> H6:...
    # head
    def Head(self, attributes:dict[str, str|int|float|None]=None) -> Head:...
    # hr
    def Hr(self, attributes:dict[str, str|int|float|None]=None) -> Hr:...
    # html
    def Html(self, attributes:dict[str, str|int|float|None]=None) -> Html: ...
    # i
    def I(self, content=None, attributes:dict[str, str|int|float|None]=None) -> I:...
    # img
    def Input(self, name: str = None, value: str = None, attributes:dict[str, str|int|float|None]=None) -> Input:...
    # input
    def Img(self, url=None, width=None, height=None, title:str= None, attributes:dict[str, str|int|float|None]=None) -> Img:...
    # input_text
    def InputText(self, name: str = None, value: str = None, placeholder:str=None, attributes:dict[str, str|int|float|None]=None) -> InputText:...
    # li
    def Li(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Li:...
    # link
    def Link(self, url=None, attributes:dict[str, str|int|float|None]=None) -> Link:...
    # meta
    def Meta(self, attributes:dict[str, str|int|float|None]=None) -> Meta:...
    # nav
    def Nav(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Nav:...
    # pre
    def Pre(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Pre:...
    # script
    def Script(self, content:str=None, attributes:dict[str, str|int|float|None]=None) -> Script:...
    # span
    def Span(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Span:...
    # style
    def Style(self, content:str=None, attributes:dict[str, str|int|float|None]=None) -> Style:...
    # table
    def Table(self, attributes:dict[str, str|int|float|None]=None) -> Table:...
    # tbody
    def Tbody(self, attributes:dict[str, str|int|float|None]=None) -> Tbody:...
    # td
    def Td(self, content=None, colspan:int=None, rowspan:int=None, attributes:dict[str, str|int|float|None]=None) -> Td:...
    # th
    def Th(self, content=None, colspan:int=None, rowspan:int=None, attributes:dict[str, str|int|float|None]=None) -> Th:...
    # thead
    def Thead(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Thead:...
    # title
    def Title(self, content=None, attributes:dict[str, str|int|float|None]=None) -> Title:...
    # tr
    def Tr(self, attributes:dict[str, str|int|float|None]=None) -> Tr:...
    # ul
    def Ul(self, attributes:dict[str, str|int|float|None]=None) -> Ul:...



from .......functions.decorators import parse_self_x_item_value_or_set_by_func
from ......types.hd import Hd


class Areas(Hd):
    @property
    def __document__(self): return self.__u__

    def add(self, name, area): self[name] = area

    @property
    @parse_self_x_item_value_or_set_by_func()
    def navbar(self): return self.__document__.root.body.inner_before(100)
    @navbar.setter
    def navbar(self, area): area.append(self.navbar)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def breadcrumb(self): return self.__document__.root.body.inner_before(90)
    @breadcrumb.setter
    def breadcrumb(self, area): area.append(self.breadcrumb)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def navigations(self): return self.__document__.root.body.inner_before(80)
    @navigations.setter
    def navigations(self, area): area.append(self.navigations)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def messages(self): return self.__document__.root.body.inner_before(70)
    @messages.setter
    def messages(self, area): area.append(self.messages)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def content(self): return self.__document__.root.body
    @content.setter
    def content(self, area): area.append(self.content)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def footer(self): return self.__document__.root.body.after(-50)
    @footer.setter
    def footer(self, area): area.append(self.footer)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def debugs(self): return self.__document__.root.body.after(-100)
    @debugs.setter
    def debugs(self, area): area.append(self.debugs)



from .settings import Settings
from ..htmdoc import Htmdoc
from ..htmdoc.areas import Areas
from ....cls_type import ClsType
from ....htmdoc.nodes.wrap import Wrap
from ....mod_type import ModType
from ...clients.client.address import Address
from ....file import File
from ....folder import Folder
from ....obj_type import ObjType
from .....types.hs import Hs
from ... import WebSystem
from .. import Website
from ....mod_path import ModPath
from ....multilinguals import Multilinguals


class Webmod(Hs):
    def __init__(self, website: Website, fullname: str): ...

    @property
    def websystem(self)-> WebSystem: ...
    """@property
    def server(self)-> Server: ..."""
    @property
    def website(self)-> Website: ...
    """ @property
    def client(self)-> Client: ...
    @property
    def database(self)-> Database: ..."""

    def webmod(self, *names: str)-> Webmod: ...
    @property
    def webmod_home(self)-> Webmod: ...
    @property
    def webmod_page_content_sign_in(self)-> Webmod: ...
    @property
    def webmod_page_content_sign_out(self)-> Webmod: ...
    @property
    def webmod_page_content_sign_up(self)-> Webmod: ...
    @property
    def webmod_page_content_about(self)-> Webmod: ...
    @property
    def webmod_page_content_contact(self)-> Webmod: ...
    @property
    def webmod_page_content_admin(self)-> Webmod: ...
    @property
    def webmod_page_layout(self)-> Webmod: ...
    @property
    def webmod_page_block_navbar(self)-> Webmod: ...
    @property
    def webmod_page_block_breadcrumb(self)-> Webmod: ...
    @property
    def webmod_page_block_navigations(self)-> Webmod: ...
    @property
    def webmod_page_block_messages(self)-> Webmod: ...
    @property
    def webmod_page_block_footer(self)-> Webmod: ...
    @property
    def webmod_page_block_debugs(self)-> Webmod: ...

    def search_children_webmods(self)-> list[Webmod]: ...
    def search_children_webmods_by_page_featur_general(self)-> list[Webmod]: ...

    def has_page_iterative(self)-> bool: ...
    def has_page_content(self)-> bool: ...
    def has_page_layout(self)-> bool: ...
    def has_page_block_navbar(self)-> bool: ...
    def has_page_block_breadcrumb(self)-> bool: ...
    def has_page_block_navigations(self)-> bool: ...
    def has_page_block_messages(self)-> bool: ...
    def has_page_block_footer(self)-> bool: ...
    def has_page_block_debugs(self)-> bool: ...
    def has_page_block_1010(self)-> bool: ...
    def has_page_block_1020(self)-> bool: ...
    def has_page_block_1030(self)-> bool: ...
    def has_page_block_1040(self)-> bool: ...
    def has_page_block_1050(self)-> bool: ...
    def has_page_block_1060(self)-> bool: ...
    def has_page_block_1070(self)-> bool: ...
    def has_page_block_1080(self)-> bool: ...
    def has_page_block_1090(self)-> bool: ...
    def has_page_block_1100(self)-> bool: ...

    @property
    def senior(self)-> Webmod | None: ...
    def child_webmod(self, *names: str): ...

    @property
    def name(self)-> str: ...
    def is_name_serialized(self)-> bool: ...
    @property
    def net_name(self)-> str: ...
    @property
    def tree_names(self)-> list[str]: ...
    @property
    def fullname(self)-> str: ...

    def is_root(self)-> bool: ...

    @property
    def tree(self)-> list[Webmod]: ...

    @property
    def mod_path(self)-> ModPath: ...
    @property
    def mod_type(self)-> ModType | None: ...
    def is_mod_exists(self)-> bool: ...
    @property
    def cls_type(self)-> ClsType | None: ...
    def is_cls_exists(self)-> bool: ...
    def has_local_method(self, name)-> bool: ...
    @property
    def obj_type(self)-> ObjType | None: ...
    def is_object_instantiated(self)-> bool: ...

    @property
    def folder(self)-> Folder: ...
    def is_folder_exists(self)-> bool: ...
    def folder_child_file(self, name: str | list[str])-> File: ...
    def folder_child_folder(self, name: str | list[str])-> Folder: ...
    @property
    def data_folder(self)-> Folder: ...
    def data_folder_child_file(self, name: str | list[str])-> File: ...
    def data_folder_child_folder(self, name: str | list[str])-> Folder: ...
    @property
    def data_configs_folder(self)-> Folder: ...
    @property
    def data_configs_file(self)-> File: ...
    @property
    def data_settings_folder(self)-> Folder: ...
    @property
    def data_settings_file(self)-> File: ...
    @property
    def data_database_tables_folder(self)-> Folder: ...
    @property
    def data_database_data_folder(self)-> Folder: ...
    @property
    def data_multilinguals_folder(self)-> Folder: ...
    def data_multilinguals_file(self, language: str)-> File: ...
    @property
    def logs_folder(self)-> Folder: ...
    @property
    def public_folder(self)-> Folder: ...
    def public_folder_child_file(self, name: str | list[str])-> File: ...
    def public_folder_child_folder(self, name: str | list[str])-> Folder: ...

    @property
    def multilinguals(self)-> Multilinguals: ...
    @property
    def ___ml_by_webmod_name___(self)-> str: ...
    def ml(self, code: str, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_name(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_abbreviation(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_description(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_link_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_page_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_breadcrumb_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...

    """@property
    def html_document(self)-> HtmlDocument: ..."""

    @property
    def htmdoc(self) -> Htmdoc: ...
    @property
    def page_areas(self)-> Areas: ...

    @property
    def page_area_content(self)-> Wrap: ...
    @property
    def page_area_navbar(self)-> Wrap: ...
    @property
    def page_area_breadcrumb(self)-> Wrap: ...
    @property
    def page_area_navigations(self)-> Wrap: ...
    @property
    def page_area_messages(self)-> Wrap: ...
    @property
    def page_area_footer(self)-> Wrap: ...
    @property
    def page_area_debugs(self)-> Wrap: ...

    def __execute__(self, name: str)-> bool: ...
    def execute_page_iterative(self)-> bool: ...
    def execute_page_content(self)-> bool: ...
    def execute_page_layout(self)-> bool: ...
    def execute_page_block_navbar(self)-> bool: ...
    def execute_page_block_breadcrumb(self)-> bool: ...
    def execute_page_block_navigations(self)-> bool: ...
    def execute_page_block_messages(self)-> bool: ...
    def execute_page_block_footer(self)-> bool: ...
    def execute_page_block_debugs(self)-> bool: ...
    def execute_page_block_1010(self)-> bool: ...
    def execute_page_block_1020(self)-> bool: ...
    def execute_page_block_1030(self)-> bool: ...
    def execute_page_block_1040(self)-> bool: ...
    def execute_page_block_1050(self)-> bool: ...
    def execute_page_block_1060(self)-> bool: ...
    def execute_page_block_1070(self)-> bool: ...
    def execute_page_block_1080(self)-> bool: ...
    def execute_page_block_1090(self)-> bool: ...
    def execute_page_block_1100(self)-> bool: ...

    def address(self, mode: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> Address: ...
    def url(self, mode: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> str: ...
    def redirect(self, mode: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None): ...

    def page_address(self, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> Address: ...
    def page_url(self, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> str: ...
    def page_redirect(self, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None): ...

    def address_by_current(self)-> Address: ...
    def url_by_current(self)-> str: ...
    def address_by_current_change_language(self, language=None)-> Address: ...
    def url_by_current_change_language(self, language=None)-> str: ...
    def address_by_current_change_pagination(self, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None)-> Address: ...
    def url_by_current_change_pagination(self, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None)-> str: ...

    def add_page_breadcrumb(self, title=None, url:str=None)-> None: ...
    def public_file_url(self, *names: str)-> str: ...
    @property
    def settings(self)-> Settings: ...
    def is_active_by_client(self)-> bool: ...


from ..... import WebSystem
from .......types.hd import Hd
from .....configs import Configs
from .....clients import Clients
from ... import Client
from .. import Request


class Headers(Hd):
    @property
    def __websystem__(self)-> WebSystem: ...
    @property
    def __clients__(self)-> Clients: ...
    @property
    def __client__(self)-> Client: ...
    @property
    def __request__(self)-> Request: ...
    @property
    def __configs__(self)-> Configs: ...

    @property
    def host(self)-> str: ...
    @property
    def connection(self)-> str: ...
    @property
    def cache_control(self)-> str: ...
    @property
    def sec_ch_ua(self)-> str: ...
    @property
    def sec_ch_ua_mobile(self)-> str: ...
    @property
    def sec_ch_ua_platform(self)-> str: ...
    @property
    def upgrade_insecure_requests(self)-> str: ...
    @property
    def user_agent(self)-> str: ...
    @property
    def accept(self)-> str: ...
    @property
    def sec_fetch_site(self)-> str: ...
    @property
    def sec_fetch_mode(self)-> str: ...
    @property
    def sec_fetch_user(self)-> str: ...
    @property
    def sec_fetch_dest(self)-> str: ...
    @property
    def accept_encoding(self)-> str: ...
    @property
    def accept_language(self)-> str: ...
    @property
    def cookie(self)-> str: ...



from .......functions.decorators import parse_self_x_item_value
from ....connections.mysql.plugins.columns import Columns
from ....connections.mysql.plugins.conditions import Conditions
from ....connections.mysql.plugins.limits import Limits
from ....connections.mysql.plugins.orders import Orders
from ......types.he import He


class Data(He):
    def __init__(self, table, columns=None, conditions=None, orders=None, limits=None):
        He.__init__(self, table)
        if columns: self.__x__['__columns__'] = columns
        if conditions: self.__x__['__conditions__'] = conditions
        if orders: self.__x__['__orders__'] = orders
        if limits: self.__x__['__limits__'] = limits

    @property
    def __database__(self): return self.__table__.__database__
    @property
    def __tables__(self): return self.__table__.__tables__
    @property
    def __table__(self): return self.__s__
    @property
    def __configs__(self): return self.__database__.configs

    @property
    @parse_self_x_item_value()
    def __columns__(self): return Columns()
    @property
    @parse_self_x_item_value()
    def __conditions__(self): return Conditions()
    @property
    @parse_self_x_item_value()
    def __limits__(self): return Limits()
    @property
    @parse_self_x_item_value()
    def __orders__(self): return Orders()



from ......types.he import He
from .... import Database
from ... import Tables
from ...table import Table
from ....configs import Configs
from ....connections.mysql.plugins.columns import Columns
from ....connections.mysql.plugins.conditions import Conditions
from ....connections.mysql.plugins.limits import Limits
from ....connections.mysql.plugins.orders import Orders


class Data(He):
    def __init__(self, table: Table, columns: Columns=None, conditions: Conditions=None, orders: Orders=None, limits: Limits=None):
        ...
    @property
    def __database__(self) -> Database: ...
    @property
    def __tables__(self) -> Tables: ...
    @property
    def __table__(self) -> Table: ...
    @property
    def __configs__(self)-> Configs: ...

    @property
    def __columns__(self) -> Columns: ...
    @property
    def __conditions__(self) -> Conditions: ...
    @property
    def __limits__(self) -> Limits: ...
    @property
    def __orders__(self) -> Orders: ...



from ...configs import Configs as ModConfigs
from ....types.he import He


class Configs(ModConfigs):
    def __init__(self, u: He): ...
    @property
    def language(self)-> str: ...
    @language.setter
    def language(self, value) -> None: ...
    @property
    def output_minimized(self)-> bool: ...
    @output_minimized.setter
    def output_minimized(self, value) -> None: ...
    @property
    def output_unspaced(self)-> bool: ...
    @output_unspaced.setter
    def output_unspaced(self, value) -> None: ...
    @property
    def output_comments(self)-> bool: ...
    @output_comments.setter
    def output_comments(self, value) -> None: ...
    @property
    def output_indentation(self)-> str: ...
    @output_indentation.setter
    def output_indentation(self, value) -> None: ...
    @property
    def title_delimiter(self)-> str: ...
    @title_delimiter.setter
    def title_delimiter(self, value) -> None: ...
    @property
    def title_ascending(self)-> bool: ...
    @title_ascending.setter
    def title_ascending(self, value) -> None: ...



# 定义断点
from hebill.builtin.system.components.css_maker import CssMaker
# 定义断点
breakpoints = {
    "sm": 576,
    "md": 768,
    "lg": 992,
    "xl": 1200,
    "xx": 1440
}
# 对应 max-width
max_widths = {
    "sm": 540,
    "md": 720,
    "lg": 960,
    "xl": 1140,
    "xx": 1408
}

prefix = 'container'
names = ['', 'sm', 'md', 'lg', 'xl', 'xx']
root_attrs = {
    '--container-padding': '15px'
}
base_attrs = {
    'width': '100%',
    'margin-left': 'auto',
    'margin-right': 'auto',
    'padding-left': 'var(--container-padding, 15px)',
    'padding-right': 'var(--container-padding, 15px)',
}

def assemble(cm: CssMaker):
    # 常量定义
    for k, v in root_attrs.items():
        cm.cls_root.atr(k, v)
    # 基础式样
    c = cm.cls(', '.join([f'.{prefix}{'-' if n else ''}{n}' for n in names]) + f', .{prefix}-fd')
    for k, v in base_attrs.items():
        c.atr(k, v)
    # 循环生成 media 规则
    for s, w in breakpoints.items():
        # 哪些 container 类适用当前断点
        idx = list(breakpoints.keys()).index(s) + 1
        aps = [f'.{prefix}{("-" + n) if n else ""}' for n in names[:idx + 1]]
        sts = ",".join(aps)
        m = cm.med(media_min_width=w)
        c = m.cls(sts)
        c.atr_px('max-width', max_widths[s])
    c = cm.cls(f'.{prefix}-fd')
    c.atr('max-width', '100%')



from ...configs import Configs as ModConfigs
from .. import WebSystem


class Configs(ModConfigs):
    def __init__(self, system: WebSystem): ...
    @property
    def __u__(self)-> WebSystem: ...

    @property
    def language(self)-> str: ...
    @language.setter
    def language(self, value) -> None: ...

    @property
    def session_expiry(self)-> int: ...
    @session_expiry.setter
    def session_expiry(self, value) -> None: ...

    @property
    def host(self)-> str: ...
    @host.setter
    def host(self, value) -> None: ...

    @property
    def port(self)-> int: ...
    @port.setter
    def port(self, value) -> None: ...

    @property
    def mysql_host(self)-> str: ...
    @mysql_host.setter
    def mysql_host(self, value) -> None: ...
    @property
    def mysql_port(self)-> int: ...
    @mysql_port.setter
    def mysql_port(self, value) -> None: ...
    @property
    def mysql_user(self)-> str: ...
    @mysql_user.setter
    def mysql_user(self, value) -> None: ...
    @property
    def mysql_password(self)-> str: ...
    @mysql_password.setter
    def mysql_password(self, value) -> None: ...
    @property
    def mysql_dbname(self)-> str: ...
    @mysql_dbname.setter
    def mysql_dbname(self, value) -> None: ...
    @property
    def mysql_prefix(self)-> str: ...
    @mysql_prefix.setter
    def mysql_prefix(self, value) -> None: ...

    @property
    def mysql_max_connections(self)-> int: ...
    @mysql_max_connections.setter
    def mysql_max_connections(self, value) -> None: ...
    @property
    def mysql_connect_timeout(self)-> int: ...
    @mysql_connect_timeout.setter
    def mysql_connect_timeout(self, value) -> None: ...

    @property
    def mysql_column_key_sn(self)-> str: ...
    @mysql_column_key_sn.setter
    def mysql_column_key_sn(self, value) -> None: ...
    @property
    def mysql_column_key_id(self)-> str: ...
    @mysql_column_key_id.setter
    def mysql_column_key_id(self, value) -> None: ...
    @property
    def mysql_column_key_az(self)-> str: ...
    @mysql_column_key_az.setter
    def mysql_column_key_az(self, value) -> None: ...
    @property
    def mysql_column_key_kw(self)-> str: ...
    @mysql_column_key_kw.setter
    def mysql_column_key_kw(self, value) -> None: ...
    @property
    def mysql_column_key_ur(self)-> str: ...
    @mysql_column_key_ur.setter
    def mysql_column_key_ur(self, value) -> None: ...
    @property
    def mysql_column_key_dc(self)-> str: ...
    @mysql_column_key_dc.setter
    def mysql_column_key_dc(self, value) -> None: ...
    @property
    def mysql_column_key_du(self)-> str: ...
    @mysql_column_key_du.setter
    def mysql_column_key_du(self, value) -> None: ...
    @property
    def mysql_data_admin_id(self)-> str: ...
    @mysql_data_admin_id.setter
    def mysql_data_admin_id(self, value) -> None: ...

    @property
    def module(self)-> str: ...
    @module.setter
    def module(self, value) -> None: ...

    @property
    def webmod_page_content_sign_in(self)-> str: ...
    @webmod_page_content_sign_in.setter
    def webmod_page_content_sign_in(self, value) -> None: ...

    @property
    def webmod_page_content_sign_out(self)-> str: ...
    @webmod_page_content_sign_out.setter
    def webmod_page_content_sign_out(self, value) -> None: ...

    @property
    def webmod_page_content_sign_up(self)-> str: ...
    @webmod_page_content_sign_up.setter
    def webmod_page_content_sign_up(self, value) -> None: ...

    @property
    def webmod_page_content_sign_reset(self)-> str: ...
    @webmod_page_content_sign_reset.setter
    def webmod_page_content_sign_reset(self, value) -> None: ...

    @property
    def webmod_page_content_admin(self)-> str: ...
    @webmod_page_content_admin.setter
    def webmod_page_content_admin(self, value) -> None: ...

    @property
    def webmod_page_content_about(self)-> str: ...
    @webmod_page_content_about.setter
    def webmod_page_content_about(self, value) -> None: ...

    @property
    def webmod_page_content_contact(self)-> str: ...
    @webmod_page_content_contact.setter
    def webmod_page_content_contact(self, value) -> None: ...

    @property
    def webmod_page_layout(self)-> str: ...
    @webmod_page_layout.setter
    def webmod_page_layout(self, value) -> None: ...

    @property
    def webmod_page_block_1010(self)-> str: ...
    @webmod_page_block_1010.setter
    def webmod_page_block_1010(self, value) -> None: ...

    @property
    def webmod_page_block_1020(self)-> str: ...
    @webmod_page_block_1020.setter
    def webmod_page_block_1020(self, value) -> None: ...

    @property
    def webmod_page_block_1030(self)-> str: ...
    @webmod_page_block_1030.setter
    def webmod_page_block_1030(self, value) -> None: ...

    @property
    def webmod_page_block_1040(self)-> str: ...
    @webmod_page_block_1040.setter
    def webmod_page_block_1040(self, value) -> None: ...

    @property
    def webmod_page_block_1050(self)-> str: ...
    @webmod_page_block_1050.setter
    def webmod_page_block_1050(self, value) -> None: ...

    @property
    def webmod_page_block_1060(self)-> str: ...
    @webmod_page_block_1060.setter
    def webmod_page_block_1060(self, value) -> None: ...

    @property
    def webmod_page_block_1070(self)-> str: ...
    @webmod_page_block_1070.setter
    def webmod_page_block_1070(self, value) -> None: ...

    @property
    def webmod_page_block_1080(self)-> str: ...
    @webmod_page_block_1080.setter
    def webmod_page_block_1080(self, value) -> None: ...

    @property
    def webmod_page_block_1090(self)-> str: ...
    @webmod_page_block_1090.setter
    def webmod_page_block_1090(self, value) -> None: ...

    @property
    def webmod_page_block_1100(self)-> str: ...
    @webmod_page_block_1100.setter
    def webmod_page_block_1100(self, value) -> None: ...

    @property
    def webmod_page_block_navbar(self)-> str: ...
    @webmod_page_block_navbar.setter
    def webmod_page_block_navbar(self, value) -> None: ...

    @property
    def webmod_page_block_breadcrumb(self)-> str: ...
    @webmod_page_block_breadcrumb.setter
    def webmod_page_block_breadcrumb(self, value) -> None: ...

    @property
    def webmod_page_block_navigations(self)-> str: ...
    @webmod_page_block_navigations.setter
    def webmod_page_block_navigations(self, value) -> None: ...

    @property
    def webmod_page_block_messages(self)-> str: ...
    @webmod_page_block_messages.setter
    def webmod_page_block_messages(self, value) -> None: ...

    @property
    def webmod_page_block_footer(self)-> str: ...
    @webmod_page_block_footer.setter
    def webmod_page_block_footer(self, value) -> None: ...

    @property
    def webmod_page_block_debugs(self)-> str: ...
    @webmod_page_block_debugs.setter
    def webmod_page_block_debugs(self, value) -> None: ...

    @property
    def html_output_minimized(self)-> str: ...
    @html_output_minimized.setter
    def html_output_minimized(self, value) -> None: ...

    @property
    def html_output_indentation(self)-> str: ...
    @html_output_indentation.setter
    def html_output_indentation(self, value) -> None: ...

    @property
    def html_output_comments(self)-> str: ...
    @html_output_comments.setter
    def html_output_comments(self, value) -> None: ...

    @property
    def html_output_unspaced(self)-> str: ...
    @html_output_unspaced.setter
    def html_output_unspaced(self, value) -> None: ...

    @property
    def html_title_delimiter(self)-> str: ...
    @html_title_delimiter.setter
    def html_title_delimiter(self, value) -> None: ...

    @property
    def html_title_ascending(self)-> bool: ...
    @html_title_ascending.setter
    def html_title_ascending(self, value) -> None: ...

    @property
    def url_root_path(self)-> str: ...
    @url_root_path.setter
    def url_root_path(self, value) -> None: ...

    @property
    def url_key_language(self)-> str: ...
    @url_key_language.setter
    def url_key_language(self, value) -> None: ...

    @property
    def url_key_mode(self)-> str: ...
    @url_key_mode.setter
    def url_key_mode(self, value) -> None: ...

    @property
    def url_key_module(self)-> str: ...
    @url_key_module.setter
    def url_key_module(self, value) -> None: ...

    @property
    def url_key_arguments(self)-> str: ...
    @url_key_arguments.setter
    def url_key_arguments(self, value) -> None: ...

    @property
    def url_key_keyword(self)-> str: ...
    @url_key_keyword.setter
    def url_key_keyword(self, value) -> None: ...

    @property
    def url_key_page(self)-> str: ...
    @url_key_page.setter
    def url_key_page(self, value) -> None: ...

    @property
    def url_key_orders(self)-> str: ...
    @url_key_orders.setter
    def url_key_orders(self, value) -> None: ...

    @property
    def url_key_filters(self)-> str: ...
    @url_key_filters.setter
    def url_key_filters(self, value) -> None: ...

    @property
    def url_key_items_per_page(self)-> str: ...
    @url_key_items_per_page.setter
    def url_key_items_per_page(self, value) -> None: ...

    @property
    def url_key_pages_per_group(self)-> str: ...
    @url_key_pages_per_group.setter
    def url_key_pages_per_group(self, value) -> None: ...

    @property
    def url_key_security(self)-> str: ...
    @url_key_security.setter
    def url_key_security(self, value) -> None: ...

    @property
    def url_key_reference(self)-> str: ...
    @url_key_reference.setter
    def url_key_reference(self, value) -> None: ...

    @property
    def pagination_items_per_page(self)-> int: ...
    @pagination_items_per_page.setter
    def pagination_items_per_page(self, value) -> None: ...

    @property
    def pagination_pages_per_group(self)-> int: ...
    @pagination_pages_per_group.setter
    def pagination_pages_per_group(self, value) -> None: ...


    @property
    def logo(self)-> str: ...
    @logo.setter
    def logo(self, value) -> None: ...



from ....types.he import He
from .table import Table


class Tables(He):
    @property
    def __database__(self): return self.__u__
    @property
    def __connection__(self): return self.__database__.connection
    @property
    def __configs__(self): return self.__database__.configs

    def table(self, name): return Table(self, name)

    def search_names(self, keyword=None, startwith=None, endwith=None):
        if keyword and startwith: keyword = self.__configs__['prefix'] + keyword
        return self.__connection__.search_tables(keyword, startwith, endwith)



import inspect
from ..file import File
from ...types.hd import Hd
from ....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name
from ...types.he import He


class Configs(Hd):
    def __init__(self, u, data=None, end_class=None, start_class=None, file=None, defaults=None):
        Hd.__init__(self, u)
        if defaults: self.___defaults___.update(defaults)
        original_data = {}
        if file:
            if isinstance(file, File):
                self.__x__['___file___'] = file
            else:
                self.__x__['___file___'] =  self.__sm__.file(file)
                data = self.___file___.parse_json()
                if isinstance(data, dict):
                    self.___defaults___.update(original_data)
        elif end_class:
            start_class = start_class if start_class else He
            if not issubclass(end_class, He): raise RuntimeError('senior_class 必须是继承自 He 的类或者实例')
            if not issubclass(end_class, start_class): raise RuntimeError('end_class 必须是继承自 start_class 的类或者实例')
            self.__x__['___file___'] = self.__sm__.cls_type(end_class).data_configs_file
            for cls in reversed(inspect.getmro(end_class)):
                if not issubclass(cls, start_class): continue
                clstyp = self.__sm__.cls_type(cls)
                if clstyp.data_configs_file.is_exists():
                    data = clstyp.data_configs_file.parse_json()
                    if isinstance(data, dict): self.___defaults___.update(data)
        self.update(self.___defaults___)
        if data: self.update(data)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___value_types___(self): return {}

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___hooks___(self): return {}
    def hook(self, name, target_configs, target_name=None): self.___hooks___[name] = (target_configs, target_name or name)

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name(module_name_striped=True)
    def ___defaults___(self): return

    def __getitem__(self, name: str):
        if name in self.___hooks___: return self.___hooks___[name][0].__getitem__(self.___hooks___[name][1])
        if name in self: return super().__getitem__(name)
        if name in self.___defaults___: return self.___defaults___[name]
        return None

    def __setitem__(self, name: str, value):
        if name in self.___hooks___: self.___hooks___[name][0].__setitem__(self.___hooks___[name][1], value)
        else:
            if name in self.___value_types___:
                pass
            super().__setitem__(name, value)

    def update(self, m, /, **kwargs):
        if isinstance(m, dict):
            for k, v in m.items(): self.__setitem__(k, v)

    def get(self, name: str): return self.__getitem__(name)
    def set(self, name: str, value): self.__setitem__(name, value)


    @property
    @parse_self_x_item_value()
    def ___file___(self): return

    def save(self):
        if self.___file___:
            self.___file___.save_json(self)



import inspect
import os
from types import ModuleType


class HeEro(Exception):
    def __init__(self, he=None, func=None, feature=None, dec=None, dec_kwargs=None, error=None):
        super().__init__(None)
        self.es = []
        self.he = None if he is None else he if isinstance(he, type) else type(he)
        self.func = func if isinstance(func, str) else None if func is None else func.__name__
        self.feature = feature
        self.dec = dec
        self.dec_kwargs = dec_kwargs or {}
        self.add(error)

    def add(self, *e):
        if e is None:
            return self
        for i in e: self.es.append(i)
        return self

    def output(self):
        def loop(e, level=0):
            pad = '  | ' * level
            lines = [
                f'{pad}####################################################################################################',
                f'{pad}*** {e.feature or "[功能说明未提供]"} ***',
            ]
            if e.he:
                lines.append(f'{pad}{e.he.__module__}.{e.he.__name__}:')
            if e.dec:
                args = ', '.join(f'{k}={v!r}' for k, v in e.dec_kwargs.items())
                lines.append(f'{pad}@{e.dec}({args})')
            if e.func:
                lines.append(f'{pad}def {e.func}(self, *args, **kwargs): ...')
            # lines.append(f'{pad}--------------------------------------------------------------------------------------------------')
            for i in e.es:
                if isinstance(i, HeEro):
                    lines.append(f'{pad}  > 子错误：')
                    lines.extend(loop(i, level + 1))
                else:
                    lines.append(f'{pad}  > {i}')
            return lines
        return '\n' + '\n'.join(loop(self))

    def __str__(self):
        return self.output()

def parse_attribute_by_path_name(pyobj: object, path_name: str):
    from operator import attrgetter
    return attrgetter(path_name)(pyobj)

def join_url(*names):
    ns = [n for i in names if (n := (i or '').strip('/'))]
    if len(ns) == 0: return ''
    if len(ns) == 1: return ns[0]
    if names[0] == '/' or names[0] == '//': return names[0] + '/'.join(['', *ns])
    return '/'.join(ns)

def join_mod_path(*names):
    return '.'.join(i for n in names if (i := (n or '').strip('.')))

def parse_pycls_name_by_mod_path(module_path: str):
    return ''.join(w.capitalize() for w in module_path.split('.')[-1].split('_'))

def parse_mod_path_by_rel_path(mod_path: str, rel_path: str):
        if not rel_path: return mod_path
        if not rel_path.startswith('.'): return join_mod_path('.'.join([mod_path, rel_path]))
        mod_path_right = rel_path.lstrip('.')
        mod_path_offset = len(rel_path) - len(mod_path_right)
        ns = mod_path.split('.')
        if len(ns) <= mod_path_offset: raise RuntimeError('"."的数量超出范围')
        return join_mod_path('.'.join([*ns[:-mod_path_offset], mod_path_right]) if mod_path_right else '.'.join(ns[:-mod_path_offset]))


def parse_mod_path_by_pycls(pycls: type, *sub_names):
    return join_mod_path(*pycls.__module__.split('.')[:-1], *sub_names)

def parse_mod_path_by_pyobj(pyobj: object, *sub_names):
    return parse_mod_path_by_pycls(pyobj.__class__, *sub_names)

def parse_dirpath_by_pymod(pymod: ModuleType):
    return os.path.dirname(pymod.__file__)

def parse_dirpath_by_pycls(pycls: type):
    return os.path.dirname(inspect.getfile(pycls))

def parse_dirpath_by_pyobj(pyobj: object):
    return parse_dirpath_by_pycls(pyobj.__class__)

def import_pymod_by_mod_path(module_path: str):
    return __import__(module_path, fromlist=[''])

def import_pycls_by_mod_path(module_path: str):
    return getattr(import_pymod_by_mod_path(f'{module_path}.core'), parse_pycls_name_by_mod_path(module_path))

def instance_pyobj_by_pycls(pycls: type, args=None, kwargs=None):
    return pycls(*(args if args else []), **(kwargs if kwargs else {}))

def instance_pyobj_by_mod_path(module_path: str, args=None, kwargs=None):
    return instance_pyobj_by_pycls(import_pycls_by_mod_path(module_path), args, kwargs)

def import_webapp_pycls_by_mod_path(module_path: str):
    return getattr(import_pymod_by_mod_path(f'{module_path}.core'), 'Core')

def timestamp_days(): return timestamp_nanoseconds() // (24 * 60 * 60 * 1000000000)
def timestamp_hours(): return timestamp_nanoseconds() // (24 * 60 * 1000000000)
def timestamp_seconds(): return timestamp_nanoseconds() // 1000000000
def timestamp_milliseconds(): return timestamp_nanoseconds() // 1000000
def timestamp_microseconds(): return timestamp_nanoseconds() // 1000
def timestamp_nanoseconds(): import time; return time.time_ns()


BASE64URL: str = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_"
def base_convert(value: str, base_from=10, base_to=32):
    if not (2 <= base_from <= 64) or not (2 <= base_to <= 64): raise RuntimeError('进制转换支持范围： 2 - 64，参数超出范围')
    if base_from == base_to: return value
    digits = BASE64URL[:base_to]
    # 转为 10 进制整数
    num = 0
    for ch in value:
        idx = digits.find(ch)
        if idx == -1: raise ValueError(f"进制转换不支持字符：'{ch}' for base {base_from}")
        num = num * base_from + idx
    if num == 0: return digits[0]
    out = []
    while num > 0:
        num, r = divmod(num, base_to)
        out.append(digits[r])
    return ''.join(reversed(out))
def uid_generater(prefix=None):
    import secrets
    return (prefix or '') + base_convert(f"{timestamp_nanoseconds()}{secrets.randbelow(10 ** 6):06d}", 10, 64)
# COMPRESS_ID 字符必须相等长度
COMPRESS_ID_STR: str = 'ST'
COMPRESS_ID_INT: str = 'IN'
COMPRESS_ID_FLT: str = 'FT'
COMPRESS_ID_DIT: str = 'DT'
COMPRESS_ID_LST: str = 'LT'
COMPRESS_ID_BOL: str = 'BL'
COMPRESS_ID_NON: str = 'NO'
def compress(data: dict | list | int | float | bool | str | None) -> str:
    import json, zlib, base64
    if isinstance(data, dict): t, d = COMPRESS_ID_DIT, json.dumps(data, separators=(',', ':')).encode()
    elif isinstance(data, list): t, d = COMPRESS_ID_LST, json.dumps(data, separators=(',', ':')).encode()
    elif isinstance(data, str): t, d = COMPRESS_ID_STR, data.encode()
    elif isinstance(data, int): t, d = COMPRESS_ID_INT, str(data).encode()
    elif isinstance(data, float): t, d = COMPRESS_ID_FLT, str(data).encode()
    elif isinstance(data, bool): t, d = COMPRESS_ID_BOL, b'1' if data else b'0'
    elif data is None: t, d = COMPRESS_ID_NON, b''
    else: raise TypeError(f"不支持压缩类型： {type(data)}")
    return f"{t}{base64.urlsafe_b64encode(zlib.compress(d, level=9)).rstrip(b'=').decode()}"
def decompress(data: str) -> dict | list | int | float | bool | str | None:
    import json, zlib, base64
    # if not data: raise ValueError("空字符串解压失败")
    if not data: return None
    t, d = data[:(l:=len(COMPRESS_ID_STR))], data[l:]
    try: r = zlib.decompress(base64.urlsafe_b64decode(d + '=' * (-len(d) % 4))).decode()
    except: raise ValueError("字符串解压失败")
    if t == COMPRESS_ID_DIT: return json.loads(r)
    if t == COMPRESS_ID_LST: return json.loads(r)
    if t == COMPRESS_ID_STR: return r
    if t == COMPRESS_ID_INT: return int(r)
    if t == COMPRESS_ID_FLT: return float(r)
    if t == COMPRESS_ID_BOL: return r == b'1'
    if t == COMPRESS_ID_NON: return None
    return None

def var2ui(value): return str(value) if isinstance(value, str) else f'"{value}"'



from .areas.core import Areas
from .caches.core import Caches
from ....htmdoc import Htmdoc as PluginHtmDoc
from ... import WebSystem
from .. import Website

class Htmdoc(PluginHtmDoc):
    def __init__(self, website: Website): ...
    @property
    def __websystem__(self)-> WebSystem: ...
    @property
    def __website__(self)-> Website: ...
    @property
    def areas(self)-> Areas: ...
    @property
    def caches(self)-> Caches: ...



from types import ModuleType

from ..cls_type import ClsType
from ..mod_type import ModType
from ...types.he import He
from ...types.hs import Hs


class ModPath(Hs):
    def __init__(self, u: He, name: str | ModuleType | type | He): ...

    @property
    def pymod(self)-> ModuleType | None: ...
    @property
    def pycls(self)-> type | None: ...

    @property
    def mod_type(self)-> ModType | None: ...
    @property
    def cls_type(self)-> ClsType | None: ...

    def rel_mod_path(self, relname: str = None)-> ModPath: ...



from ..he import He
from ..hf.number import Number


class Hi(He, int, Number):
    def __new__(cls, u: He, number: str | float | int): ...
    def __init__(self, u: He, number: str | float | int): ...



from ..core import CssMaker
from ....types.he import He


class Cls(He):
    def __init__(self, u, name: str): ...

    @property
    def __u__(self)-> CssMaker: ...

    @property
    def name(self)-> str: ...
    @property
    def atrs(self)-> dict[str, str]: ...

    def atr(self, name:str=None, value:str=None): ...
    def atr_px(self, name:str=None, value:str|int|float=None): ...



from datetime import tzinfo

from . import Hf
from ..he import He
from ..hi import Hi


class Number(He, float):
    CN_INTEGER_0_TO_9 = '零壹贰叁肆伍陆柒捌玖'
    CN_POINTS_UNITS = '角分厘毫丝'
    CN_DECIMAL_4S_UNITS = '万亿兆京垓秭穰沟涧正载极'
    CN_DECIMAL_UNITS = '拾佰仟'

    CN_CURRENCY_UNIT = "圆"
    CN_POINT_NAME = '点'
    CN_ONLY_NAME = '整'
    CN_CNY_NAME = '人民币'
    CN_USD_NAME = '美元'
    CN_EUR_NAME = '欧元'
    CN_MINUS = '负'

    EN_CNY_NAME = 'Say Chinese Yuan'
    EN_USD_NAME = 'Say U.S.Dollar'
    EN_EUR_NAME = 'Say Euro'
    EN_ONLY_NAME = 'Only'
    EN_POINT_NAME = 'Point'
    EN_AND_NAME = 'and'
    EN_MINUS = 'Minus'

    EN_INTEGER_0_TO_9 = ['Zero', 'One', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight', 'Nine']
    EN_INTEGER_1NS = ['Ten', 'Eleven', 'Twelve', 'Thirteen', 'Fourteen', 'Fifteen', 'Sixteen', 'Seventeen',
                      'Eighteen', 'Nineteen']
    EN_INTEGER_2NS = ['Twenty', 'Thirty', 'Forty', 'Fifty', 'Sixty', 'Seventy', 'Eighty', 'Ninety']
    EN_HUNDRED_NAME = 'Hundred'
    EN_DECIMAL_3S_UNITS = ['Thousand', 'Million', 'Billion', 'Trillion', 'Quadrillion', 'Quintillion', 'Sextillion',
                           'Septillion', 'Octillion', 'Nonillion', 'Decillion']
    EN_POINTS_UNITS = ['Dime', 'Cent', 'Mill']

    WEIGHT_UNIT_KGS = 'Kgs'
    CN_WEIGHT_UNIT_KGS = '公斤'
    EN_WEIGHT_UNIT_KGS = 'Kilograms'

    LENGTH_UNIT_MM = 'mm'
    CN_LENGTH_UNIT_MM = '毫米'
    EN_LENGTH_UNIT_MM = 'Millimetres'

    LENGTH_UNIT_M = 'm'
    CN_LENGTH_UNIT_M = '米'
    EN_LENGTH_UNIT_M = 'Metres'

    LENGTH_UNIT_CM = 'cm'
    CN_LENGTH_UNIT_CM = '厘米'
    EN_LENGTH_UNIT_CM = 'Centimeters'


    @property
    def is_int(self) -> bool: ...
    def is_positive(self) -> bool: ...
    def is_negative(self) -> bool: ...
    def is_zero(self) -> bool: ...

    def round(self, digits: int = 0)-> Hf | Hi: ...
    def ceil(self, digits: int = 0)-> Hf | Hi: ...
    def floor(self, digits: int = 0)-> Hf | Hi: ...
    @property
    def integer_part(self)-> str: ...
    @property
    def ___integer_part_breaks___(self)-> dict[int, list[str]]: ...
    def integer_part_breaks(self, step: int)-> list[str]: ...
    @property
    def ___integer_part_strict_breaks___(self)-> dict[int, list[str]]: ...
    def integer_part_strict_breaks(self, step: int)-> list[str]: ...
    @property
    def integer_part_myriad_breaks(self)-> list[str]: ...
    @property
    def integer_part_myriad_strict_breaks(self)-> list[str]: ...
    @property
    def integer_part_thousand_breaks(self)-> list[str]: ...
    @property
    def integer_part_thousand_strict_breaks(self)-> list[str]: ...
    @property
    def decimal_part(self)-> str: ...
    ####################################################################################################
    def round_format(self, digits: int = 0, force_digits: bool = None, integer_breaks: int = 3)-> str: ...
    ####################################################################################################
    def year(self, tz: tzinfo = None)-> int: ...
    def month(self, tz: tzinfo = None)-> int: ...
    def day(self, tz: tzinfo = None)-> int: ...
    def hour(self, tz: tzinfo = None)-> int: ...
    def minute(self, tz: tzinfo = None)-> int: ...
    def second(self, tz: tzinfo = None)-> int: ...
    def format_yy(self, tz: tzinfo = None)-> str: ...
    def format_m(self, tz: tzinfo = None)-> str: ...
    def format_d(self, tz: tzinfo = None)-> str: ...
    def format_h(self, tz: tzinfo = None)-> str: ...
    def format_i(self, tz: tzinfo = None)-> str: ...
    def format_s(self, tz: tzinfo = None)-> str: ...
    def format_yyyy(self, tz: tzinfo = None)-> str: ...
    def format_mm(self, tz: tzinfo = None)-> str: ...
    def format_dd(self, tz: tzinfo = None)-> str: ...
    def format_hh(self, tz: tzinfo = None)-> str: ...
    def format_ii(self, tz: tzinfo = None)-> str: ...
    def format_ss(self, tz: tzinfo = None)-> str: ...

    def format_yymd(self, tz: tzinfo = None, delimiter='-')-> str: ...
    def format_his(self, tz: tzinfo = None, delimiter=':')-> str: ...
    def format_yymdhis(self, tz: tzinfo = None, delimiter=' ', date_delimiter='-', time_delimiter=':')-> str: ...
    def format_yyyymmdd(self, tz: tzinfo = None, delimiter='-')-> str: ...
    def format_hhiiss(self, tz: tzinfo = None, delimiter=':')-> str: ...
    def format_yyyymmddhhiiss(self, tz: tzinfo = None, delimiter=' ', date_delimiter='-', time_delimiter=':')-> str: ...


    ####################################################################################################
    # 以下数字转化为大写
    ####################################################################################################
    @property
    def cn_decimal_part(self) -> str: ...
    """def ___integer_myriad_breaks___(self) -> list: ...
    def ___integer_myriad_strict_breaks___(self) -> list: ...
    def ___integer_thousand_breaks___(self) -> list: ...
    def ___integer_thousand_strict_breaks___(self) -> list: ..."""
    def _capitalized_myriad_breaks_number_to_cn(self, group_sn, num_sn): ...
    @property
    def cn_integer_part(self) -> str: ...
    @property
    def cn_number(self) -> str: ...
    @property
    def cn_money(self) -> str: ...
    @property
    def cn_cny(self) -> str: ...
    @property
    def cn_usd(self) -> str: ...
    @property
    def cn_eur(self) -> str: ...
    @property
    def en_decimal_part(self) -> str: ...
    def _capitalized_en_by_thousands_breaks_add_and(self, group_sn, num_sn): ...
    def _capitalized_en_by_thousands_breaks(self, group_sn): ...
    def _capitalized_en_by_thousands_breaks_2(self, group_sn): ...
    def _capitalized_en_by_thousands_breaks_1(self, group_sn): ...
    @property
    def en_integer_part(self) -> str: ...
    @property
    def en_number(self) -> str: ...
    @property
    def en_money(self) -> str: ...
    @property
    def en_cny(self) -> str: ...
    @property
    def en_usd(self) -> str: ...
    @property
    def en_eur(self) -> str: ...



from ......functions.decorators import parse_self_x_item_value
from ...nodes.tag import Tag


class Script(Tag):
    def __init__(self, u, url=None, code=None, attributes:dict[str, str|int|float|None]=None):
        Tag.__init__(self, u, None, attributes)
        if url: self.attributes['src'] = url
        self.__x__['content'] = self.script(code)

    @property
    @parse_self_x_item_value()
    def content(self): return


from ..... import WebSystem
from .......types.hd import Hd
from .....configs import Configs
from .....clients import Clients
from ... import Client
from .. import Request


class Arguments(Hd):
    @property
    def __websystem__(self)-> WebSystem: ...
    @property
    def __clients__(self)-> Clients: ...
    @property
    def __client__(self)-> Client: ...
    @property
    def __request__(self)-> Request: ...
    @property
    def __configs__(self)-> Configs: ...



import os.path
from ..file_folder import FileFolder


class File(FileFolder):
    def __init__(self, u, path: str):
        super().__init__(u, path)

    @property
    def base_name(self): return self.name.rsplit('.', 1)[0]

    @property
    def extension_name(self): return self.name.rsplit('.', 1)[1] if '.' in self.name else ''

    def is_exists(self): return os.path.isfile(self)
    def is_unexists(self): return not self.is_exists()
    def parse_json(self):
        import json
        try:
            with open(self, 'r', encoding='utf-8') as f:
                return json.load(f)
        except Exception:
            return None

    def read_content(self):
        try:
            with open(self, 'r', encoding='utf-8') as file: return file.read()
        except Exception: return None

    def read_lines(self):
        result = []
        try:
            with open(self, 'r', encoding='utf-8') as file:
                for line in file: result.append(line)
            return result
        except Exception: return None

    def save_content(self, content: str):
        try:
            with open(self, 'w', encoding='utf-8') as file: return file.write(content)
        except Exception: return False

    def read_bytes(self):
        try:
            with open(self, 'rb') as file: return file.read()
        except Exception: return None

    def save_json(self, data: dict):
        import json
        try:
            with open(self, 'w', encoding='utf-8') as file:
                json.dump(data, file, ensure_ascii=False, indent=4)
                return True
        except Exception: return False

    def copy_to(self, path):
        if isinstance(path, File): file = path
        elif isinstance(path, str): file = self.__s__.components.file(path)
        else: return False
        if self != file and self.is_exists():
            try:
                import shutil
                shutil.copy(self, file)
                return True
            except Exception: return False
        return False

    def copy_from(self, path):
        if isinstance(path, str):
            file = self.__s__.components.file(path)
        elif isinstance(path, File):
            file = path
        else:
            return False
        return file.copy_to(self)

    def save_io(self, bytes_io):
        # 确保 io 的指针在开头
        try:
            bytes_io.seek(0)
            with open(self, "wb") as f: f.write(bytes_io.read())
            return True
        except Exception: return False




from ...__data__ import Data


class PaginationSelect(Data):
    @property
    def columns(self): return self.__columns__
    @property
    def conditions(self): return self.__conditions__
    @property
    def orders(self): return self.__orders__

    def execute(self, page_number, items_per_page):
        self.__limits__.set(page_number * (items_per_page - 1), items_per_page)
        return self.__queries__.select_from_table(self.__table__.fullname, self.columns, self.conditions, self.orders, self.__limits__)



from ...__data__ import Data


class Update(Data):
    @property
    def conditions(self): return self.__conditions__
    def execute(self, data): return self.__queries__.update_table(self.__table__.fullname, data, self.conditions)



from ...configs import Configs as ModConfigs
from ....types.he import He


class Configs(ModConfigs):
    def __init__(self, u: He): ...
    @property
    def connection(self)-> str: ...
    @connection.setter
    def connection(self, value) -> None: ...
    @property
    def host(self)-> str: ...
    @host.setter
    def host(self, value) -> None: ...
    @property
    def port(self)-> int: ...
    @port.setter
    def port(self, value) -> None: ...
    @property
    def user(self)-> str: ...
    @user.setter
    def user(self, value) -> None: ...
    @property
    def password(self)-> str: ...
    @password.setter
    def password(self, value) -> None: ...
    @property
    def dbname(self)-> str: ...
    @dbname.setter
    def dbname(self, value) -> None: ...
    @property
    def prefix(self)-> str: ...
    @prefix.setter
    def prefix(self, value) -> None: ...
    @property
    def max_connections(self)-> int: ...
    @max_connections.setter
    def max_connections(self, value) -> None: ...
    @property
    def connect_timeout(self)-> int: ...
    @connect_timeout.setter
    def connect_timeout(self, value) -> None: ...
    @property
    def column_key_sn(self)-> str: ...
    @column_key_sn.setter
    def column_key_sn(self, value) -> None: ...
    @property
    def column_key_id(self)-> str: ...
    @column_key_id.setter
    def column_key_id(self, value) -> None: ...
    @property
    def column_key_az(self)-> str: ...
    @column_key_az.setter
    def column_key_az(self, value) -> None: ...
    @property
    def column_key_kw(self)-> str: ...
    @column_key_kw.setter
    def column_key_kw(self, value) -> None: ...
    @property
    def column_key_ur(self)-> str: ...
    @column_key_ur.setter
    def column_key_ur(self, value) -> None: ...
    @property
    def column_key_dc(self)-> str: ...
    @column_key_dc.setter
    def column_key_dc(self, value) -> None: ...
    @property
    def column_key_du(self)-> str: ...
    @column_key_du.setter
    def column_key_du(self, value) -> None: ...
    @property
    def data_admin_id(self)-> str: ...
    @data_admin_id.setter
    def data_admin_id(self, value) -> None: ...



from ..he import He


class Hs(He, str):
    def __new__(cls, u, string: str):
        obj = str.__new__(cls, string)
        obj.u = u
        return obj

    def __init__(self, u, string: str):
        He.__init__(self, u)



from ......functions.decorators import parse_self_x_item_value
from ..wrap import Wrap


class Tag(Wrap):
    def __init__(self, u, name=None, content=None, attributes:dict[str, str|int|float|None]=None):
        Wrap.__init__(self, u, content)
        self.attributes.update(attributes or {})
        if name: self.___name___ = name
        else:
            mro = type(self).__mro__
            self.___name___ = mro[mro.index(Tag) - 1].__name__.lower()


    @staticmethod
    def is_wrap(): return False
    @staticmethod
    def is_tag(): return True

    @property
    def name(self): return self.___name___

    @property
    def attributes(self): return self.___attributes___

    def inner_before(self, sn:int): return self.___inner_before___(sn)
    @property
    def inner_before1(self): return self.___anchor___(1)
    @property
    def inner_before2(self): return self.___anchor___(2)
    @property
    def inner_before3(self): return self.___anchor___(3)
    @property
    def inner_before4(self): return self.___anchor___(4)
    @property
    def inner_before5(self): return self.___anchor___(5)
    def inner_after(self, sn:int):  return self.___inner_after___(sn)
    @property
    def inner_after1(self): return self.___anchor___(-1)
    @property
    def inner_after2(self): return self.___anchor___(-2)
    @property
    def inner_after3(self): return self.___anchor___(-3)
    @property
    def inner_after4(self): return self.___anchor___(-4)
    @property
    def inner_after5(self): return self.___anchor___(-5)

    @property
    @parse_self_x_item_value()
    def output_pairable(self): return
    @property
    @parse_self_x_item_value()
    def output_brealable(self): return



from hebill.builtin.system.components.css_maker import CssMaker

breakpoints = {
    "sm": 576,
    "md": 768,
    "lg": 992,
    "xl": 1200,
    "xx": 1440
}
col_qn = 12
col_ws = [f"{i/col_qn*100:.6f}%" for i in range(1, col_qn+1)]

def assemble(css_maker: CssMaker):
    # Grid 基础变量
    css_maker.cls_root.atr('--row-gap', '15px')
    css_maker.cls_root.atr('--cell-gap', '15px')

    # .grid 容器
    c = css_maker.cls('.grid')
    c.atr('width', '100%')

    # .row
    r = css_maker.cls('.row')
    r.atr('display', 'flex')
    r.atr('flex-wrap', 'wrap')
    r.atr('margin-left', 'calc(-1 * var(--cell-gap))')
    r.atr('margin-right', 'calc(-1 * var(--cell-gap))')
    r.atr('row-gap', 'var(--row-gap)')

    # .cell
    cell = css_maker.cls('.cell')
    cell.atr('padding-left', 'var(--cell-gap)')
    cell.atr('padding-right', 'var(--cell-gap)')
    cell.atr('box-sizing', 'border-box')
    cell.atr('flex', '0 0 auto')

    # 默认列宽
    for i, w in enumerate(col_ws, 1):
        c = css_maker.cls(f'.cell-{i}')
        c.atr('flex-basis', w)
        c.atr('max-width', w)

    # 响应式列
    for bp_name, bp_width in breakpoints.items():
        m = css_maker.med(media_min_width=bp_width)
        for i, w in enumerate(col_ws, 1):
            c = m.cls(f'.{bp_name}\\:cell-{i}')
            c.atr('flex-basis', w)
            c.atr('max-width', w)

    # 响应式 row-gap / cell-gap（可选）
    for bp_name, bp_width in breakpoints.items():
        m = css_maker.med(media_min_width=bp_width)
        r = m.cls('.row')
        r.atr('--row-gap', '15px')   # 可按断点调整
        r.atr('--cell-gap', '15px')  # 可按断点调整



from ...nodes.tag import Tag


class Link(Tag):
    __hd_tag_pairable__ = False
    def __init__(self, u, url=None, attributes:dict[str, str|int|float|None]=None):
        Tag.__init__(self, u, None, None, attributes)
        self.attributes['href'] = url



import os.path

from .....functions.builtins import join_url, decompress
from .....functions.decorators import parse_self_x_item_value_or_set_by_func, parse_self_x_item_value, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name
from ....types.hd import Hd


class Website(Hd):
    def __init__(self, websystem, client):
        super().__init__(websystem)
        self.__x__['client'] = client

    @property
    @parse_self_x_item_value_or_set_by_func()
    def websystem(self): return self.__u__
    @property
    @parse_self_x_item_value()
    def client(self): return

    """@property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def html_document(self): return"""

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def htmdoc(self): return

    @property
    def favicon_ico_url_path(self): return join_url('/', self.websystem.configs.url_root_path, 'favicon.ico')

    def webmod(self, *names: str):
        ns = [n for i in names if (n:=i.strip('.'))]
        from .webmod import Webmod
        return Webmod(self, '.'.join(ns) if ns else '')

    @property
    def webroot(self): return self.webmod()

    def address(self, mode=None, module=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.client.address(mode, module, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security)
    def url(self, mode=None, module=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address(mode, module, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).output()
    def redirect(self, mode=None, module=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address(mode, module, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).redirect()

    def response(self):
        si = self.__s__.__i__
        got_path_names = [i for i in self.client.request.path.strip(' /').split('/') if i]
        print('排查', 100, 'got_path_names', got_path_names)
        rel_path_names = [i for i in self.websystem.configs.url_root_path.strip(' /').split('/') if i]
        print('排查', 200, 'rel_path_names', rel_path_names)
        if len(got_path_names) < len(rel_path_names) or got_path_names[:len(rel_path_names)] != rel_path_names:
            return self.websystem.server.make_response_404()
        if len(rel_path_names) > 0: act_path_names = got_path_names[:len(rel_path_names)]
        else: act_path_names = got_path_names
        print('排查', 300, 'rel_path_names', rel_path_names)
        def response_favicon():
            if (i:= self.webroot.folder.child_file('logo.ico')).is_exists(): return self.websystem.server.make_response_file(i)
            if (i:= si.folder.child_file('logo.ico')).is_exists(): return self.websystem.server.make_response_file(i)
            return self.websystem.server.make_response_404()
        def response_cdn():
            if (i:=si.cdn_folder.child_file(*act_path_names[1:])).is_exists(): return self.websystem.server.make_response_file(i)
            return self.websystem.server.make_response_404()
        def response_pub():
            if (i:=self.webroot.folder.child_file(*act_path_names[1:])).is_exists(): return self.websystem.server.make_response_file(i)
            return self.websystem.server.make_response_404()
        def parse_requested_parameters():
            self.client.request.address.language = act_path_names[0]
            try: a = decompress(act_path_names[1])
            except: a = {}
            if not isinstance(a, dict): a = {}
            self.client.request.address.language = a.get(self.websystem.configs.url_key_language, self.websystem.configs.language)
            self.client.request.address.module = a.get(self.websystem.configs.url_key_module, '')
            self.client.request.address.arguments = a.get(self.websystem.configs.url_key_arguments, {})
            self.client.request.address.keyword = a.get(self.websystem.configs.url_key_keyword, None)
            self.client.request.address.page = a.get(self.websystem.configs.url_key_page, 1)
            self.client.request.address.orders = a.get(self.websystem.configs.url_key_orders, {})
            self.client.request.address.filters = a.get(self.websystem.configs.url_key_filters, {})
            self.client.request.address.items_per_page = a.get(self.websystem.configs.url_key_items_per_page, self.websystem.configs.pagination_items_per_page)
            self.client.request.address.pages_per_group = a.get(self.websystem.configs.url_key_pages_per_group, self.websystem.configs.pagination_pages_per_group)
            self.client.request.address.reference = a.get(self.websystem.configs.url_key_reference, '')
            self.client.request.address.security = a.get(self.websystem.configs.url_key_security, '')
            self.client.request.arguments.update(a.get(self.websystem.configs.url_key_arguments, {}))

        def response_pag():
            self.websystem.server.__deubg_info__(f'请求页面[{self.client.request.method}]: [{self.client.request.address.module or '/'}] <=> {self.client.request.url}')
            # 运行模板
            m = self.webmod(self.client.request.address.module)
            mod_lay = m.settings['page_layout'] or self.websystem.configs['webmod_page_layout']
            if mod_lay == '#': mod_lay = str(m)
            if not self.webmod(mod_lay).execute_page_layout():
                self.webroot.execute_page_layout()
            # 运行内容，错误返回错误模块
            # 权限限制： 401，页面找不到：404，运行错误：500，建设中：501
            power = True
            def execute_page():
                if m.is_folder_exists():
                    if power:
                        if m.is_mod_exists():
                            if m.is_cls_exists():
                                if m.is_object_instantiated():
                                    if (result := m.obj_type.pyobj.page_content()) is not None: return result
                                    """try: return rst if (rst:=m.pyobject.page_content()) is not None else None
                                    except: self.webmod(self.websystem.configs.webmod_page_block_1100).execute_page_block_1100()"""
                                    return None
                                self.webmod(self.websystem.configs.webmod_page_block_1090).execute_page_block_1090()
                                return None
                            self.webmod(self.websystem.configs.webmod_page_block_1080).execute_page_block_1080()
                            return None
                        self.webmod(self.websystem.configs.webmod_page_block_1070).execute_page_block_1070()
                        return None
                    self.webmod(self.websystem.configs.webmod_page_block_1060).execute_page_block_1060()
                    return None
                self.webmod(self.websystem.configs.webmod_page_block_1050).execute_page_block_1050()
                return None

            executed = execute_page()
            if executed is not None: return executed
            return self.websystem.server.make_response_text_html(self.htmdoc.output())
        print('排查', 1100, 'act_path_names', act_path_names)
        if len(act_path_names) == 0: act_path_names = [si.WEBMOD_PAG]
        print('排查', 1200, 'act_path_names', act_path_names)
        if act_path_names[0] == 'favicon.ico': return response_favicon()
        print('排查', 1300, 'act_path_names', act_path_names)
        match act_path_names[0]:
            case si.WEBMOD_CDN:
                print('排查', 2100, 'act_path_names', act_path_names)
                return response_cdn()
            case si.WEBMOD_PUB:
                print('排查', 2200, 'act_path_names', act_path_names)
                return response_pub()
            case si.WEBMOD_AJX | si.WEBMOD_FIL | si.WEBMOD_LAY | si.WEBMOD_JSN | si.WEBMOD_PAG | si.WEBMOD_STM:
                print('排查', 2300, 'act_path_names', act_path_names)
                parse_requested_parameters()
                match act_path_names[0]:
                    case si.WEBMOD_AJX: # TODO
                        print('排查', 3100, 'act_path_names', act_path_names)
                        return self.websystem.server.make_response_text_plain('这里是请求的 Ajax 內容')
                    case si.WEBMOD_FIL: # TODO
                        print('排查', 3200, 'act_path_names', act_path_names)
                        return self.websystem.server.make_response_text_plain('这里是请求的 File 內容')
                    case si.WEBMOD_LAY: # TODO
                        print('排查', 3300, 'act_path_names', act_path_names)
                        return self.websystem.server.make_response_text_plain('这里是请求的 Layout 內容')
                    case si.WEBMOD_JSN: # TODO
                        print('排查', 3400, 'act_path_names', act_path_names)
                        return self.websystem.server.make_response_text_plain('这里是请求的 Json 內容')
                    case si.WEBMOD_PAG:
                        print('排查', 3500, 'act_path_names', act_path_names)
                        return response_pag()
                    case si.WEBMOD_STM: # TODO
                        print('排查', 3600, 'act_path_names', act_path_names)
                        return self.websystem.server.make_response_text_plain('这里是请求的 Steam 內容')
            case _:
                print('排查', 3400, 'act_path_names', act_path_names)
                return self.websystem.server.make_response_404()



from decimal import Decimal
from ..he import He
from ..hf.number import Number

class Hm(He, Number, Decimal):
    def __new__(cls, u, number: str | float | int):
        """
        __new__ 负责创建实例
        - str 类型：去掉逗号和空格，转 Decimal
        - int 类型：直接 Decimal
        - float 类型：先转 str 再转 Decimal 避免浮点误差
        """
        if isinstance(number, str):
            try: number = Decimal(number.replace(',', '').replace(' ', ''))
            except Exception: raise RuntimeError('实例化 Hm 实例参数 number 必须支持 Decimal(number)。')
        elif isinstance(number, int):
            number = Decimal(number)
        elif isinstance(number, float):
            number = Decimal(str(number))
        else:
            raise RuntimeError('实例化 Hm 实例参数 number 只支持 str | float | int 。')
        # 使用 Decimal.__new__ 创建实例
        obj = Decimal.__new__(cls, number)
        obj.u = u
        return obj

    def __init__(self, u, number: str | float | int):
        """
        __init__ 初始化 He 部分
        """
        He.__init__(self, u)



from ..components.configs import Configs as ModConfigs
from ...functions.decorators import parse_self_item_value, set_self_item_value_by_setter_with_agr


class Configs(ModConfigs):
    def __init__(self, s):
      super().__init__(s, None, s.__class__, s.__class__)

    @property
    @parse_self_item_value()
    def language(self): return
    @language.setter
    @set_self_item_value_by_setter_with_agr(value_type='lan')
    def language(self, value): pass

    @property
    @parse_self_item_value()
    def debug_prefix(self): return
    @debug_prefix.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def debug_prefix(self, value): pass

    @property
    @parse_self_item_value()
    def debug_printable(self): return
    @debug_printable.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def debug_printable(self, value): pass

    @property
    @parse_self_item_value()
    def debug_saveable(self): return
    @debug_saveable.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def debug_saveable(self, value): pass



from ......types.he import He
from .quantity import Quantity
from .drop import Drop
from .update import Update
from .select import Select
from .pagination_select import PaginationSelect


class Data(He):
    def __init__(self, u):
        He.__init__(self, u)
    @property
    def __database__(self): return self.__tables__.__database__
    @property
    def __tables__(self): return self.__table__.__tables__
    @property
    def __table__(self): return self.__u__
    @property
    def __configs__(self): return self.__database__.configs
    @property
    def __queries__(self): return self.__database__.queries

    def quantity_obj(self, conditions=None): return Quantity(self.__table__, conditions=conditions)
    def quantity(self, conditions=None): return self.quantity_obj(conditions).execute()
    def drop_obj(self, conditions=None): return Drop(self.__table__, conditions=conditions)
    def drop(self, conditions=None): return self.drop_obj(conditions).execute()
    def insert(self, data, user=None, keywords=None): return self.__queries__.insert_into_table(self.__table__.fullname, data, user, keywords)
    def update_obj(self, conditions=None): return Update(self.__table__, conditions=conditions)
    def update(self, data, conditions=None): return self.update_obj(conditions).execute(data)
    def select_obj(self, columns=None, conditions=None, orders=None, limits=None): return Select(self.__table__, columns, conditions, orders, limits)
    def select(self, columns=None, conditions=None, orders=None, limits=None): return self.select_obj(columns, conditions, orders, limits).execute()
    def pagination_select_obj(self, columns=None, conditions=None, orders=None): return PaginationSelect(self.__table__, columns, conditions, orders)
    def pagination_select(self, page_number, items_per_page, columns=None, conditions=None, orders=None): return self.pagination_select_obj(columns, conditions, orders).execute(page_number, items_per_page)



from ......types.hd import Hd
from .. import Htmdoc
from .....htmdoc.nodes.wrap import Wrap


class Areas(Hd):
    @property
    def __document__(self)-> Htmdoc: ...

    def add(self, name: str, area: Wrap): ...

    def get(self, name: str)-> Wrap | None: ...

    @property
    def content(self)-> Wrap: ...
    @content.setter
    def content(self, area: Wrap): ...

    @property
    def navbar(self)-> Wrap: ...
    @navbar.setter
    def navbar(self, area: Wrap): ...

    @property
    def breadcrumb(self)-> Wrap: ...
    @breadcrumb.setter
    def breadcrumb(self, area: Wrap): ...

    @property
    def navigations(self)-> Wrap: ...
    @navigations.setter
    def navigations(self, area: Wrap): ...

    @property
    def footer(self)-> Wrap: ...
    @footer.setter
    def footer(self, area: Wrap): ...

    @property
    def messages(self)-> Wrap: ...
    @messages.setter
    def messages(self, area: Wrap): ...

    @property
    def debugs(self)-> Wrap: ...
    @debugs.setter
    def debugs(self, area: Wrap): ...



from ... import Htmdoc
from ...nodes.tag import Tag
from ...nodes.wrap import Wrap


class Img(Tag):
    attribute_align_values: list = ...

    def __init__(self, u: Htmdoc | Wrap, url: str = None, width: int = None, height: int = None, title: str = None, attributes:dict[str, str|int|float|None]=None): ...




from ...nodes.tag import Tag


class Hr(Tag):
    __hd_tag_pairable__ = False
    def __init__(self, u, attributes:dict[str, str|int|float|None]=None):
        Tag.__init__(self, u, None, attributes)



from ... import Htmdoc
from ...nodes.code import Code
from ...nodes.tag import Tag
from ...nodes.wrap import Wrap


class Script(Tag):
    def __init__(self, u: Htmdoc | Wrap, code: str = None, attributes:dict[str, str|int|float|None]=None): ...
    @property
    def content(self)-> Code: ...



from .....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func, \
    parse_dict_item_value_or_set_by_instance_he_by_module_name
from ....types.he import He


class Med(He):
    @staticmethod
    def ___make_name___(media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None):
        medias = []
        if media_min_width: medias.append(f'(min-width: {media_min_width}px)')
        if media_max_width: medias.append(f'(max-width: {media_max_width}px)')
        if media_min_height: medias.append(f'(min-height: {media_min_height}px)')
        if media_max_height: medias.append(f'(max-height: {media_max_height}px)')
        return ' and '.join(medias)

    def __init__(self, u, media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None):
        He.__init__(self, u)
        medias = []
        if media_min_width: medias.append(f'(min-width: {media_min_width}px)')
        if media_max_width: medias.append(f'(max-width: {media_max_width}px)')
        if media_min_height: medias.append(f'(min-height: {media_min_height}px)')
        if media_max_height: medias.append(f'(max-height: {media_max_height}px)')
        self.__x__['name'] = ' and '.join(medias)

    @property
    @parse_self_x_item_value()
    def name(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def clses(self): return {}
    @parse_dict_item_value_or_set_by_instance_he_by_module_name(dict_attr_name='clses', dict_item_key_by_arg=0, module_name='.cls')
    def cls(self, name:str): return

    def __str__(self):
        ls = [f'@media {self.name} {{']
        for _, c in self.clses.items():
            ls.append(c.__str__())
        ls.append(f'}}')
        return '\n'.join(ls)



from .....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func
from ....types.hd import Hd


class Package(Hd):
    def __init__(self, multilingual, language):
        super().__init__(multilingual)
        self.__x__['language'] = language
        if self.__u__.___tree_cls_types___:
            for cls_typ in reversed(self.__u__.___tree_cls_types___):
                file = cls_typ.data_multilinguals_file(self.language)
                if file.is_exists():
                    data = file.parse_json()
                    if isinstance(data, dict):
                        self.update(data)
        else:
            if self.file.is_exists():
                data = self.file.parse_json()
                if isinstance(data, dict):
                    self.update(data)

    @property
    @parse_self_x_item_value()
    def language(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def file(self): return self.__u__.folder.child_file(f'{self.language}.json')

    def load(self): self.update(self.file.parse_json())

    def read(self, code): return self.get(code)

    def ml(self, code, replace=None, return_code=False):
        return self.read(code) or replace or (code if return_code else None)

    def save(self, code, traslation=None):
        if isinstance(code, str) and traslation is not None:
            self[code] = traslation
        elif isinstance(code, dict):
            self.update(code)
        self.file.save_json(self)



from ...configs import Configs as PluCfg
from ...file import File
from ...folder import Folder
from ...mod_type import ModType


class Configs(PluCfg):
    @property
    def mainpy(self)-> str: ...
    @mainpy.setter
    def mainpy(self, value: str): ...

    @property
    def website_module(self)-> str: ...
    @website_module.setter
    def website_module(self, value: str): ...

    @property
    def exe_name(self)-> str: ...
    @exe_name.setter
    def exe_name(self, value: str): ...

    @property
    def show_console(self)-> bool: ...
    @show_console.setter
    def show_console(self, value: bool): ...

    @property
    def one_file(self)-> bool: ...
    @one_file.setter
    def one_file(self, value: bool): ...

    @property
    def one_dir(self)-> bool: ...
    @one_dir.setter
    def one_dir(self, value: bool): ...

    @property
    def pack_overwrite(self)-> bool: ...
    @pack_overwrite.setter
    def pack_overwrite(self, value: bool): ...

    @property
    def root_folders(self)-> list[Folder]: ...
    @root_folders.setter
    def root_folders(self, value: list[Folder]): ...

    @property
    def root_modules(self)-> list[ModType]: ...
    @root_modules.setter
    def root_modules(self, value: list[ModType]): ...

    @property
    def dist_path(self)-> Folder: ...
    @dist_path.setter
    def dist_path(self, value: Folder): ...

    @property
    def build_path(self)-> Folder: ...
    @build_path.setter
    def build_path(self, value: Folder): ...

    @property
    def spec_path(self)-> Folder: ...
    @spec_path.setter
    def spec_path(self, value: Folder): ...

    @property
    def comd_path(self)-> Folder: ...
    @comd_path.setter
    def comd_path(self, value: Folder): ...

    @property
    def icon_file(self)-> File: ...
    @icon_file.setter
    def icon_file(self, value: File): ...

    @property
    def version_file(self)-> File: ...
    @version_file.setter
    def version_file(self, value: File): ...



from ...nodes.tag import Tag


class Input(Tag):
    __hd_tag_pairable__ = False
    def __init__(self, u, name: str = None, value: str = None, attributes:dict[str, str|int|float|None]=None):
        if isinstance(attributes, dict):
            if 'type' not in attributes:
                attributes['type'] = 'text'
        else:
            attributes = {'type': 'text'}
        if name is not None: attributes['name'] = name
        if value is not None: attributes['value'] = value
        Tag.__init__(self, u, None, attributes)



from ....types.he import He
from colorama import Fore, Style


class Output(He):
    def print(self, content=None, error=None, comment=None, file=None, prefix=None, color=None, bgcolor=None, bright=None):
        text = Fore.BLUE + self.__u__.make_prefix(prefix)
        if bgcolor: text += bgcolor
        if color: text += color
        if bright: text += Style.BRIGHT
        text += str(content)
        if error: text += f'{Fore.RED + Style.DIM} # {error}'
        if comment: text += f'{Fore.WHITE + Style.DIM} #{comment}'
        if file: text += f'{Fore.WHITE + Style.DIM} #File:"{file}",'
        text += Style.RESET_ALL
        return print(text)

    def primary(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, '\033[38;2;200;200;200m')
    def secondary(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, Fore.BLUE)
    def success(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, Fore.GREEN)
    def warning(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, Fore.YELLOW)
    def danger(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, Fore.RED)
    def info(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, Fore.CYAN)
    def comment(self, content=None, error=None, comment=None, file=None, prefix=None): return self.print(content, error, comment, file, prefix, Fore.WHITE)



from types import ModuleType
from .file import File
from ...functions.decorators import instance_he_by_module_name, parse_self_x_item_value_or_set_by_func
from ..types.he import He


class Components(He):
    _a0 = {'arg0':'__s__'}

    @instance_he_by_module_name(**_a0)
    def cls_type(self, pycls: He | type): return
    @instance_he_by_module_name(**_a0)
    def configs(self, data=None, end_class=None, start_class=None, file=None): return
    @instance_he_by_module_name(**_a0)
    def css_maker(self, file: str | File = None): return

    @instance_he_by_module_name(**_a0)
    def database(self, host=None, user=None, password=None, dbname=None, port=None, prefix=None, connection=None, configs:dict=None): return
    @instance_he_by_module_name(**_a0)
    def debugs(self): return

    @instance_he_by_module_name(**_a0)
    def exe_packer(self, u, mainpy:str, website_module:str=None, one_file:bool=False, one_dir:bool=False, show_console:bool=False, pack_overwrite:bool=False): return

    @instance_he_by_module_name(**_a0)
    def file(self, path: str): return
    @instance_he_by_module_name(**_a0)
    def folder(self): return

    @instance_he_by_module_name(**_a0)
    def image(self, source: str): return

    @instance_he_by_module_name(**_a0)
    def htmdoc(self): return

    @instance_he_by_module_name(**_a0)
    def mod_path(self, name: str | ModuleType | type | He): return
    @instance_he_by_module_name(**_a0)
    def mod_type(self, pymod: ModuleType | type | He): return
    @instance_he_by_module_name(**_a0)
    def multilinguals(self, end_pycls=None, start_pycls=None, folder=None, hooked_configs=None, hooked_config_name=None): return

    @instance_he_by_module_name(**_a0)
    def obj_type(self, pyobj: He): return

    @instance_he_by_module_name(**_a0)
    def web_system(self, module: str, host: str = None, port: int = None, db_host: str=None, db_user: str=None, db_password: str=None, db_dbname: str=None, db_port: int=None, db_prefix: str=None, configs:dict=None): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def image_icon_yangs(self): from .image import IconYangs; return IconYangs(self.__s__)
    @property
    @parse_self_x_item_value_or_set_by_func()
    def image_icon_hebill(self): from .image import IconHebill; return IconHebill(self.__s__)


from .......functions.decorators import parse_self_x_item_value_or_set_by_func
from ......types.hd import Hd


class Caches(Hd):
    @property
    @parse_self_x_item_value_or_set_by_func()
    def breadcrumb(self): from .breadcrumb import Breadcrumb; return Breadcrumb()
    @property
    @parse_self_x_item_value_or_set_by_func()
    def navbar(self): from .navbar import Navbar; return Navbar()



from ......functions.builtins import timestamp_seconds
from ......functions.decorators import parse_self_item_value, \
    parse_self_item_value_or_set_by_instance_he_by_module_name
from .....types.hd import Hd


class Client(Hd):
    def __init__(self, u, uid):
        super().__init__(u)
        self['uid'] = uid
        self['expiry'] = self.__configs__['session_expiry'] + timestamp_seconds()

    @property
    def __websystem__(self): return self.__clients__.__websystem__
    @property
    def __clients__(self): return self.__u__
    @property
    def __configs__(self): return self.__websystem__.configs

    @property
    @parse_self_item_value()
    def uid(self): return
    @property
    @parse_self_item_value()
    def expiry(self): return

    @property
    @parse_self_item_value_or_set_by_instance_he_by_module_name()
    def request(self): return

    def hook_session(self, session: dict): self['session'] = session

    @property
    @parse_self_item_value()
    def session(self): return

    @parse_self_item_value_or_set_by_instance_he_by_module_name()
    def address(self, mode=None, module=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return
    def url(self, mode=None, module=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address(mode, module, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).output()
    def redirect(self, mode=None, module=None, arguments=None, keyword=None, page=None, orders=None, filters=None, items_per_page=None, pages_per_group=None, reference=None, language=None, security=None): return self.address(mode, module, arguments, keyword, page, orders, filters, items_per_page, pages_per_group, reference, language, security).redirect()



from types import ModuleType
from hebill.builtin.system.types.he import He
from hebill.builtin.system.types.hs import Hs
from ..cls_type import ClsType
from ..file import File
from ..folder import Folder
from ..mod_path import ModPath
from ..multilinguals import Multilinguals


class ModType(Hs):
    def __init__(self, u: He, pymod: ModuleType | type | He): ...

    @property
    def pymod(self)-> ModuleType: ...
    @property
    def pycls(self)-> type | None: ...

    @property
    def mod_path(self)-> ModPath: ...
    @property
    def cls_type(self)-> ClsType | None: ...

    def rel_mod_path(self, rel_path=None)-> ModPath: ...
    def rel_mod_type(self, rel_path=None)-> ModType | None: ...

    @property
    def folder(self)-> Folder: ...

    @property
    def data_folder(self)-> Folder: ...
    @property
    def data_configs_folder(self)-> Folder: ...
    @property
    def data_configs_file(self)-> File: ...
    @property
    def data_database_tables_folder(self)-> Folder: ...
    def data_database_table_file(self, name: str)-> File: ...
    @property
    def data_database_data_folder(self)-> Folder: ...
    def data_database_data_file(self, name: str)-> File: ...
    @property
    def data_multilinguals_folder(self)-> Folder: ...
    def data_multilinguals_file(self, language: str)-> File: ...

    @property
    def public_folder(self)-> Folder: ...

    @property
    def logs_folder(self)-> Folder: ...

    @property
    def multilinguals(self)-> Multilinguals: ...
    def ml(self, code: str, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_name(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_abbreviation(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_description(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_link_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_page_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_breadcrumb_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...



from ...functions.decorators import instance_he_by_module_name
from ..types.he import He


class Types(He):
    _a0 = {'arg0':'__s__'}

    @instance_he_by_module_name(**_a0)
    def hd(self, data: dict=None): return
    @instance_he_by_module_name(**_a0)
    def he(self, data: dict=None): return
    @instance_he_by_module_name(**_a0)
    def hf(self, number: str | float | int): return
    @instance_he_by_module_name(**_a0)
    def hi(self, number: str | float | int): return
    @instance_he_by_module_name(**_a0)
    def hl(self, data: list=None): return
    @instance_he_by_module_name(**_a0)
    def hm(self, data: list=None): return
    @instance_he_by_module_name(**_a0)
    def hs(self, string: str): return





from ........functions.decorators import parse_self_item_value
from .......types.hd import Hd


class Headers(Hd):
    @property
    def __websystem__(self): return self.__clients__.__websystem__
    @property
    def __clients__(self): return self.__client__.__clients__
    @property
    def __client__(self): return self.__request__.__client__
    @property
    def __request__(self): return self.__u__
    @property
    def __configs__(self): return self.__websystem__.configs

    @property
    @parse_self_item_value()
    def host(self): return
    @property
    @parse_self_item_value()
    def connection(self): return
    @property
    @parse_self_item_value()
    def cache_control(self): return
    @property
    @parse_self_item_value()
    def sec_ch_ua(self): return
    @property
    @parse_self_item_value()
    def sec_ch_ua_mobile(self): return
    @property
    @parse_self_item_value()
    def sec_ch_ua_platform(self): return
    @property
    @parse_self_item_value()
    def upgrade_insecure_requests(self): return
    @property
    @parse_self_item_value()
    def user_agent(self): return
    @property
    @parse_self_item_value()
    def accept(self): return
    @property
    @parse_self_item_value()
    def sec_fetch_site(self): return
    @property
    @parse_self_item_value()
    def sec_fetch_mode(self): return
    @property
    @parse_self_item_value()
    def sec_fetch_user(self): return
    @property
    @parse_self_item_value()
    def sec_fetch_dest(self): return
    @property
    @parse_self_item_value()
    def accept_encoding(self): return
    @property
    @parse_self_item_value()
    def accept_language(self): return
    @property
    @parse_self_item_value()
    def cookie(self): return




from ..components.folder import Folder
from ..components.mod_path import ModPath
from ..components.mod_type import ModType
from ..types.he.i import I as HI


class I(HI):
    DIRNAM_DATA:str = ...
    DIRNAM_DATA_CONFIGS:str = ...
    DIRNAM_DATA_SETTINGS:str = ...
    DIRNAM_DATA_DATABASE_TABLES:str = ...
    DIRNAM_DATA_DATABASE_DATA:str = ...
    DIRNAM_DATA_MULTILINGALS:str = ...
    DIRNAM_LOGS:str = ...
    DIRNAM_PUBLIC:str = ...

    FILNAM_CONFIGS:str = ...
    FILNAM_SETTINGS:str = ...

    MLKEY_NAME:str = ...
    MLKEY_ABBR:str = ...
    MLKEY_TITLE:str = ...
    MLKEY_DESC:str = ...
    MLKEY_LINK_TITLE:str = ...
    MLKEY_PAGE_TITLE:str = ...
    MLKEY_BDCD_TITLE:str = ...

    LAN_CN:str = ...
    LAN_GB:str = ...
    LANS: list[str] = ...
    WEBMOD_CDN:str = ...
    WEBMOD_PUB:str = ...
    WEBMOD_AJX:str = ...
    WEBMOD_FIL:str = ...
    WEBMOD_LAY:str = ...
    WEBMOD_JSN:str = ...
    WEBMOD_PAG:str = ...
    WEBMOD_STM:str = ...
    WEBMODS: list[str] = ...
    DATCOL_BIN:str = ...
    DATCOL_CHA:str = ...
    DATCOL_DOB:str = ...
    DATCOL_FLT:str = ...
    DATCOL_INT:str = ...
    DATCOL_TXT:str = ...
    DATCOL_TIN:str = ...
    DATCOL_VCH:str = ...
    DATCOLS: list[str] = ...


    @property
    def builtin_folder(self)-> Folder: ...
    @property
    def builtin_mod_path(self)-> ModPath: ...
    @property
    def builtin_mod_type(self)-> ModType: ...
    @property
    def hebill_folder(self)-> Folder: ...
    @property
    def hebill_mod_path(self)-> ModPath: ...
    @property
    def hebill_mod_type(self)-> ModType: ...
    @property
    def project_folder(self)-> Folder: ...
    @property
    def project_mod_path(self)-> ModPath: ...
    @property
    def project_mod_type(self)-> ModType: ...
    @property
    def cdn_folder(self)-> Folder: ...
    @property
    def website_folder(self)-> Folder: ...
    @property
    def website_mod_path(self)-> ModPath: ...
    @property
    def website_mod_type(self)-> ModType: ...



from .menu import Menu
from .dropdown import Dropdown


class Menus(list):
    def add_menu(self, title, url=None):
        menu = Menu(title, url)
        self.append(menu)
        return menu

    def add_dropdown(self, title):
        dropdown = Dropdown(title)
        self.append(dropdown)
        return dropdown



from ......types.he import He
from .... import DatabaseMysql
from ... import Tables
from .. import Table
from ....configs import Configs
from ....queries import Queries
from ....queries.plugins.columns import Columns
from ....queries.plugins.conditions import Conditions
from ....queries.plugins.limits import Limits
from ....queries.plugins.orders import Orders
from .quantity import Quantity
from .drop import Drop
from .update import Update
from .select import Select
from .pagination_select import PaginationSelect


class Data(He):
    def __init__(self, u: Tables, name: str): ...
    @property
    def __database__(self)-> DatabaseMysql: ...
    @property
    def __tables__(self)-> Tables: ...
    @property
    def __table__(self)-> Table: ...
    @property
    def __configs__(self)-> Configs: ...
    @property
    def __queries__(self)-> Queries: ...

    def quantity_obj(self, conditions: Conditions = None)-> Quantity: ...
    def quantity(self, conditions: Conditions = None)-> bool: ...
    def drop_obj(self, conditions: Conditions = None)-> Drop: ...
    def drop(self, conditions: Conditions = None)-> bool: ...
    def insert(self, data: dict, user: str = None, keywords: list | str = None)-> bool: ...
    def update_obj(self, conditions: Conditions = None)-> Update: ...
    def update(self, data: dict, conditions: Conditions = None)-> bool: ...
    def select_obj(self, columns: Columns = None, conditions: Conditions = None, orders: Orders = None, limits: Limits = None)-> Select: ...
    def select(self, columns: Columns = None, conditions: Conditions = None, orders: Orders = None, limits: Limits = None)-> bool: ...
    def pagination_select_obj(self, columns: Columns = None, conditions: Conditions = None, orders: Orders = None)-> PaginationSelect: ...
    def pagination_select(self, page_number: int, items_per_page: int, columns: Columns = None, conditions: Conditions = None, orders: Orders = None)-> bool: ...



from .hd import Hd
from .he import He
from .hf import Hf
from .hi import Hi
from .hl import Hl
from .hs import Hs


class Types(He):
    def hd(self, data: dict=None)-> Hd: ...
    def he(self, data: dict=None)-> He: ...
    def hf(self, number: str | float | int)-> Hf: ...
    def hm(self, number: str | float | int)-> Hm: ...
    def hi(self, number: str | float | int)-> Hi: ...
    def hl(self, data: list=None)-> Hl: ...
    def hs(self, string: str)-> Hs: ...



from ..he import He
from ..hf.number import Number


class Hi(He, int, Number):
    def __new__(cls, u, number: str | float | int):
        if isinstance(number, str):
            try: number = float(number.replace(',', ''))
            except: raise RuntimeError('实例化 Hi 实例参数 number 必须支持 float(number)。')
        if not isinstance(number, float | int):
            raise RuntimeError('实例化 Hi 实例参数 number 只支持 str | float | int 。')
        if number != int(number):
            raise RuntimeError('实例化 Hi 实例参数 number 转化为数字后必须为整型 。')
        # noinspection PyTypeChecker
        obj = int.__new__(cls, int(number))
        obj.u = u
        return obj

    def __init__(self, u, number: str | float | int):
        He.__init__(self, u)



from ..types.he.i import I as HI
from ...functions.decorators import parse_self_x_item_value_or_set_by_func


class I(HI):
    DIRNAM_DATA:str = '.data'
    DIRNAM_DATA_CONFIGS:str = 'configs'
    DIRNAM_DATA_SETTINGS:str = 'settings'
    DIRNAM_DATA_DATABASE_TABLES:str = 'database_tables'
    DIRNAM_DATA_DATABASE_DATA:str = 'database_data'
    DIRNAM_DATA_MULTILINGALS:str = 'multilinguals'
    DIRNAM_LOGS:str = '.logs'
    DIRNAM_PUBLIC:str = '.public'

    FILNAM_CONFIGS:str = 'configs.json'
    FILNAM_SETTINGS:str = 'settings.json'

    MLKEY_NAME:str = 'NAME'
    MLKEY_ABBR:str = 'ABBR'
    MLKEY_TITLE:str = 'TITLE'
    MLKEY_DESC:str = 'DESP'
    MLKEY_LINK_TITLE:str = 'LINK_TITLE'
    MLKEY_PAGE_TITLE:str = 'PAGE_TITLE'
    MLKEY_BDCD_TITLE:str = 'BDCD_TITLE'

    LAN_CN = 'cn'
    LAN_GB = 'gb'
    LANS: list[str] = [LAN_CN, LAN_GB]
    WEBMOD_CDN = 'cdn'
    WEBMOD_PUB = 'pub'
    WEBMOD_AJX = 'ajax'
    WEBMOD_FIL = 'file'
    WEBMOD_LAY = 'layout'
    WEBMOD_JSN = 'json'
    WEBMOD_PAG = 'page'
    WEBMOD_STM = 'steam'
    WEBMODS: list[str] = [WEBMOD_CDN, WEBMOD_PUB, WEBMOD_AJX, WEBMOD_FIL, WEBMOD_LAY, WEBMOD_JSN, WEBMOD_PAG, WEBMOD_STM]
    DATCOL_BIN = 'bigint'
    DATCOL_CHA = 'char'
    DATCOL_DOB = 'double'
    DATCOL_FLT = 'float'
    DATCOL_INT = 'int'
    DATCOL_TXT = 'text'
    DATCOL_TIN = 'tinyint'
    DATCOL_VCH = 'varchar'
    DATCOLS: list[str] = [DATCOL_BIN, DATCOL_CHA, DATCOL_DOB, DATCOL_FLT, DATCOL_INT, DATCOL_TXT, DATCOL_TIN, DATCOL_VCH]

    @property
    def builtin_folder(self): return self.folder.parent
    @property
    @parse_self_x_item_value_or_set_by_func()
    def builtin_mod_path(self): return self.mod_path.rel_mod_path('.')
    @property
    def builtin_mod_type(self): return self.builtin_mod_path.mod_type
    @property
    def hebill_folder(self): return self.builtin_folder.parent
    @property
    @parse_self_x_item_value_or_set_by_func()
    def hebill_mod_path(self): return self.builtin_mod_path.rel_mod_path('.')
    @property
    def hebill_mod_type(self): return self.hebill_mod_path.mod_type
    @property
    def project_folder(self): return self.hebill_folder.parent
    @property
    @parse_self_x_item_value_or_set_by_func()
    def project_mod_path(self): return self.hebill_mod_path.rel_mod_path('.')
    @property
    def project_mod_type(self): return self.project_mod_path.mod_type
    @property
    @parse_self_x_item_value_or_set_by_func()
    def cdn_folder(self): return self.hebill_folder.child_folder('cdn')
    @property
    @parse_self_x_item_value_or_set_by_func()
    def website_folder(self): return self.hebill_folder.child_folder('website')
    @property
    @parse_self_x_item_value_or_set_by_func()
    def website_mod_path(self): return self.hebill_mod_path.rel_mod_path('hebill.website')
    @property
    def website_mod_type(self): return self.hebill_folder.child_folder('website')




from ..he import He


class Hd(He, dict):
    def __init__(self, u, data: dict=None):
        He.__init__(self, u)
        dict.__init__(self)
        if data: self.update(data)

    def __ds__(self, name, value): self[name] = value

    def __getitem__(self, name: str):
        return super().__getitem__(name) if name in self else None

    def get(self, name):
        return self.__getitem__(name)



from ....types.he import He
from .. import Database


class Connection(He):
    @property
    def __u__(self)-> Database: ...

    def is_connected(self, recheck: bool = False)-> bool: ...
    @property
    def queried_sqls(self)-> list[str]: ...
    @property
    def last_queried_sql(self)-> str | None: ...
    @property
    def queried_times(self)-> int: ...

    def query(self, sql: str): ...

    def search_tables(self, keyword: str = None, startwith: bool = None, endwith: bool = None)-> list[str]: ...

    def is_table_exists(self, table: str)-> bool: ...

    def create_table(self, table)-> bool: ...
    def delete_table(self, table)-> bool: ...
    def update_table_auto_increment(self, table, start=0)-> bool: ...

    def search_table_column_structures(self, table: str, keyword: str = None, startwith: bool = None, endwith: bool = None)-> list[dict]: ...
    def search_table_columns(self, table: str, keyword: str = None, startwith: bool = None, endwith: bool = None)-> list[str]: ...
    def create_table_column(self, table: str, column: str, datatype: str = None, length: int = 0, unique: bool = False, nullable: bool = False)-> bool: ...
    def set_table_column_quniue(self, table: str, column: str)-> bool: ...

    def search_table_data_multiple(self, sql: str)-> list[dict]: ...



from hebill import Hebill

if __name__ == '__main__':
    he = Hebill()
    cm = he.components.css_maker('css.css')

    from dev.colors import assemble
    assemble(cm)

    from dev.container import assemble
    assemble(cm)

    from dev.grid import assemble
    assemble(cm)

    # container 响应宽度
    cm.save(True)


class Orders(dict):
    def __init__(self, orders=None):
        dict().__init__(self)
        self.adds(orders or {})

    def add(self, column, by=None):
        if column:
            self[column] = "DESC" if by is not None and by is not False and by.upper() == "DESC" else "ASC"

    def adds(self, orders):
        if orders:
            for k, v in orders.items():
                self.add(k, v)

    def output(self):
        if len(self) >= 1:
            return f" ORDER BY {", ".join(f'`{k}` {v}' for k, v in self.items())}"
        return ""



import types

from ..cls_type import ClsType
from ..file import File
from ..folder import Folder
from ..mod_path import ModPath
from ..mod_type import ModType
from ..multilinguals import Multilinguals
from ..web_system.webapp import Webapp
from ...types.he import He
from ...types.hs import Hs


class ObjType(Hs):
    def __init__(self, u: He, pyobj: He | type): ...

    @property
    def pymod(self)-> types.ModuleType: ...
    @property
    def pycls(self)-> type: ...
    @property
    def pyobj(self)-> Webapp: ...

    @property
    def mod_name(self)-> ModPath: ...
    @property
    def mod_type(self)-> ModType: ...
    @property
    def cls_type(self)-> ClsType: ...
    @property
    def tree_pyclsses(self)-> list[type]: ...
    @property
    def tree_cls_types(self)-> list[ClsType]: ...

    def has_function(self, name: str)-> bool: ...
    def has_local_function(self, name: str)-> bool: ...

    @property
    def folder(self)-> Folder: ...
    @property
    def data_folder(self)-> Folder: ...
    @property
    def data_configs_folder(self)-> Folder: ...
    @property
    def data_configs_file(self)-> File: ...
    @property
    def data_multilinguals_folder(self)-> Folder: ...
    def data_multilinguals_file(self, language: str)-> File: ...
    @property
    def logs_folder(self)-> Folder: ...
    @property
    def data_database_tables_folder(self)-> Folder: ...
    @property
    def data_database_data_folder(self)-> Folder: ...

    @property
    def multilinguals(self)-> Multilinguals: ...
    def ml(self, code: str, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_name(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_abbreviation(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_description(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_link_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_page_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...
    def ml_breadcrumb_title(self, language: str = None, replace: str = None, return_code: bool = None)-> str | None: ...


from ....functions.decorators import parse_self_x_item_value_or_set_by_instance_he_by_module_name, \
    parse_self_x_item_value_or_set_by_func
from ...types.he import He


class Debugs(He):
    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name(search_from_self=True)
    def configs(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name(search_from_self=True)
    def input(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name(search_from_self=True)
    def output(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___prefixes___(self): return []

    @property
    def prefixes(self): return [self.configs.prefix, *self.___prefixes___] if self.configs.prefix else self.___prefixes___

    def add_prefix(self, prefix):
        if prefix :self.___prefixes___.append(prefix)

    def make_prefix(self, prefix):
        prefixes = []
        if self.configs.prefix: prefixes.append(self.configs.prefix)
        if isinstance(prefix, str) and prefix:prefixes.append(prefix)
        elif isinstance(prefix, list | tuple): prefixes.extend([i for i in prefix if i])
        if self.configs.prefix and self.___prefixes___: return '/'.join(f'[{i}]' for i in prefixes) + '/:>'
        return '[HEBILL]/:>'



import os
from ....functions.decorators import parse_self_x_item_value_or_set_by_func
from ...types.hs import Hs


class FileFolder(Hs):
    def __init__(self, u, path: str):
        super().__init__(u, path)

    @property
    def name(self): return os.path.basename(self)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def parent(self): return self.__sm__.folder(os.path.dirname(self))
    def brother_file(self, name): return self.parent.child_file(name)
    def brother_folder(self, name): return self.parent.child_folder(name)



class Limits(dict):
    def __init__(self, start=None, quantity=None):
        dict().__init__(self)
        self.set(start or 0, quantity or 0)

    def set(self, start, quantity):
        self.start = start
        self.quantity = quantity

    @property
    def start(self): return self["start"]
    @start.setter
    def start(self, value: int): self["start"] = value or 0

    @property
    def quantity(self): return self["quantity"]
    @quantity.setter
    def quantity(self, value: int): self["quantity"] = value or 0

    def output(self):
        if self.quantity <= 0:
            return ""
        return f' LIMIT {self.start}, {self.quantity}'



class Limits(dict):
    def __init__(self, start: int = None, quantity: int = None): ...

    def set(self, start: int, quantity: int)-> None: ...

    @property
    def start(self)-> int: ...
    @start.setter
    def start(self, value: int) -> None: ...

    @property
    def quantity(self)-> int: ...
    @quantity.setter
    def quantity(self, value: int) -> None: ...

    def output(self) -> str: ...



from .configs import Configs
from ..file import File
from ..folder import Folder
from ..mod_type import ModType
from ...types.he import He


class ExePacker(He):
    def __init__(self, u: He, mainpy:str, website_module:str=None, one_file:bool=False, one_dir:bool=False, show_console:bool=False, pack_overwrite:bool=False): ...

    @property
    def configs(self)-> Configs: ...
    def add_root_folder(self, folder: str | Folder): ...
    def ___check_root_module___(self, module: str | ModType)-> ModType | None: ...
    def add_root_module(self, module: str | ModType): ...

    def __output_commands__(self)-> list[str]: ...
    def pack(self): ...
    def output_command(self) -> str: ...
    def save_command(self, file: str|File=None)-> None: ...



from ...nodes.tag import Tag


class A(Tag):
    def __init__(self, u, title=None, url:str = None, attributes:dict[str, str|int|float|None]=None):
        (attributes := attributes or {})['href'] = url
        Tag.__init__(self, u, None, title, attributes)



from ...__data__ import Data
from .....queries.plugins.columns import Columns
from .....queries.plugins.conditions import Conditions
from .....queries.plugins.orders import Orders


class PaginationSelect(Data):
    @property
    def columns(self) -> Columns: ...
    @property
    def conditions(self) -> Conditions: ...
    @property
    def orders(self) -> Orders: ...

    def execute(self, page_number: int, items_per_page: int) -> list[dict]: ...




import logging
import os
import uuid
from flask import Flask, request, Response, abort, redirect, session, send_file
from ...server import Server


class FlaskServer(Server):
    # 运行start后怎么实现子线程运行，不影响主线程继续运行
    def __deubg_warning__(self, content=None, error=None, comment=None, file=None): self.__s__.debugs.output.warning(content, error, comment, file, [self.__configs__.module, f'{self.__configs__['host']}:{self.__configs__['port']}', request.remote_addr])
    def __deubg_info__(self, content=None, error=None, comment=None, file=None): self.__s__.debugs.output.info(content, error, comment, file, [self.__configs__.module, f'{self.__configs__['host']}:{self.__configs__['port']}', request.remote_addr])
    def make_response_text_html(self, code: str): return Response(code, mimetype='text/html')
    def make_response_text_plain(self, text: str): return Response(text, mimetype='text/html')
    def make_response_text_css(self, code: str): return Response(code, mimetype='text/html')
    def make_response_text_csv(self, code: str): return Response(code, mimetype='text/html')
    def make_response_text_markdown(self, code: str): return Response(code, mimetype='text/html')
    def make_response_text_xml(self, code: str): return Response(code, mimetype='text/html')

    def make_response_redirect(self, url): return redirect(url)
    def make_response_file(self, file, as_attachment=False, mimetype=None, download_name=None):
        msg = f'请求文件[GET]: {file} <=> {request.url}'
        if os.path.isfile(file):
            self.__deubg_info__(msg)
            return send_file(file, as_attachment=as_attachment, mimetype=mimetype, download_name=download_name)
        self.__deubg_warning__(msg)
        return self.make_response_404()
    def make_response_404(self): return abort(404)

    def run(self):
        logging.getLogger('werkzeug').setLevel(logging.ERROR)
        app = Flask(__name__)
        app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 86400
        app.secret_key = uuid.uuid4().hex
        @app.route('/', defaults={'path': ''}, methods=['GET', 'POST'])
        @app.route('/<path:path>', methods=['GET', 'POST'])
        def home(path):
            if not 'hebill' in session: session['hebill'] = {}
            if not 'id' in session: session['hebill']['id'] = str(uuid.uuid4())
            client = self.__websystem__.clients.get(session['hebill']['id'])
            client.hook_session(session['hebill'])
            client.request.headers.update({
                'host': request.headers.get('Host'),
                'connection': request.headers.get('Connection'),
                'cache_control': request.headers.get('Cache-Control'),
                'sec_ch_ua': request.headers.get('Sec-Ch-Ua'),
                'sec_ch_ua_mobile': request.headers.get('Sec-Ch-Ua-Mobile'),
                'sec_ch_ua_platform': request.headers.get('Sec-Ch-Ua-Platform'),
                'upgrade_insecure_requests': request.headers.get('Upgrade-Insecure-Requests'),
                'user_agent': request.headers.get('User-Agent'),
                'accept': request.headers.get('Accept'),
                'sec_fetch_site': request.headers.get('Sec-Fetch-Site'),
                'sec_fetch_mode': request.headers.get('Sec-Fetch-Mode'),
                'sec_fetch_user': request.headers.get('Sec-Fetch-User'),
                'sec_fetch_dest': request.headers.get('Sec-Fetch-Dest'),
                'accept_encoding': request.headers.get('Accept-Encoding'),
                'accept_language': request.headers.get('Accept-Language'),
                'cookie': request.headers.get('Cookie'),
            })
            client.request.form.update(request.form.to_dict())
            client.request.update({
                'method': request.method,
                'url': request.url,
                'path': request.path,
                'remote_addr': request.remote_addr,
            })
            # return 'Hello World'
            from ...website import Website
            return Website(self.__websystem__, client).response()

        self.__s__.debugs.output.info(f'网站开始运行：http://{'127.0.0.1' if self.__configs__['host'] == '0.0.0.0' else self.__configs__['host']}:{self.__configs__['port']}')
        app.run(host=self.__configs__['host'], port=self.__configs__['port'], debug=False, threaded=False)



from .. import He
from ....components.cls_type import ClsType
from ....components.folder import Folder
from ....components.mod_path import ModPath
from ....components.mod_type import ModType
from ....components.obj_type import ObjType


class I:
    def __init__(self, he):
        self.__x: dict = ...

    @property
    def __he__(self)-> He: ...
    @property
    def __x__(self)-> dict: ...

    @property
    def id(self)-> int: ...
    @property
    def mod_path(self)-> ModPath: ...
    @property
    def mod_type(self)-> ModType: ...
    @property
    def cls_type(self)-> ClsType: ...
    @property
    def obj_type(self)-> ObjType: ...

    @property
    def folder(self)-> Folder: ...



class Conditions(list):
    def __init__(self, logic=None, is_root=True):
        list.__init__(self)
        self._logic = 'AND' if logic is None or logic.upper == 'AND' else 'OR'
        self._is_root = is_root

    @property
    def is_root(self): return self._is_root
    @property
    def logic(self): return self._logic
    @logic.setter
    def logic(self, value): self._logic = 'AND' if value.upper == 'AND' else 'OR'
    def set_logic_and(self): self._logic = 'AND'
    def set_logic_or(self): self._logic = 'OR'

    def add(self, column, value, judge=None, logic=None, startwith=None, endwith=None):
        from .condition.core import Condition
        self.append(Condition(column, value, judge, logic, startwith, endwith))
        return self[-1]

    def add_and_equal(self, column, value): return self.add(column, value, "=", "AND")
    def add_or_equal(self, column, value): return self.add(column, value, "=", "OR")
    def add_and_not_equal(self, column, value): return self.add(column, value, "!=", "AND")
    def add_and_more_than(self, column, value): return self.add(column, value, ">", "AND")
    def add_and_not_more_than(self, column, value): return self.add(column, value, "<=", "AND")
    def add_and_less_than(self, column, value): return self.add(column, value, "<", "AND")
    def add_and_not_less_than(self, column, value): return self.add(column, value, ">=", "AND")

    def add_and_like(self, column, value, startwith=None, endwith=None):
        return self.add(column, value, "LIKE", "AND", startwith, endwith)

    def add_or_like(self, column, value, startwith=None, endwith=None):
        return self.add(column, value, "LIKE", "OR", startwith, endwith)

    def add_conditions(self, logic=None):
        self.append(Conditions(logic, False))
        return self[-1]

    def add_conditions_and(self): return self.add_conditions("AND")
    def add_conditions_or(self): return self.add_conditions("OR")

    def output(self, logic_bridge_requeried=False):
        s = ""
        for ss in self:
            s += ss.output(bool(s))
        if not s.strip():
            return ""

        if self.is_root:
            if not logic_bridge_requeried:
                return " " + s
            return f" {self.logic} ({s})"
        return f' WHERE{s}' if s else ""



from .menus import Menus


class Navbar(dict):
    def __init__(self):
        super().__init__()
        self.update({
            'logo': '',
            'title': '',
            'menus': Menus(),
        })

    @property
    def logo(self): return self.get('logo')
    @logo.setter
    def logo(self, value): self['logo'] = value

    @property
    def title(self): return self.get('title')
    @title.setter
    def title(self, value): self['title'] = value

    @property
    def menus(self): return self.get('menus')


from .i import I
from ... import System
from ...components import Components as SystemComponents
from ...configs import Configs as SystemConfigs
from ...debugs import Debugs as SystemDebugs
from ...types import Types as SystemTypes


class He:
    def __init__(self, u: He):
        self.__x: dict = ...

    @property
    def __i__(self)-> I: ...
    @property
    def __id__(self)-> int: ...
    @property
    def __x__(self)-> dict: ...

    @property
    def __s__(self)-> System: ...
    @property
    def __sm__(self)-> SystemComponents: ...
    @property
    def __sc__(self)-> SystemConfigs: ...
    @property
    def __sd__(self)-> SystemDebugs: ...
    @property
    def __st__(self)-> SystemTypes: ...

    @property
    def __u__(self)-> He | None: ...

    @property
    def __l__(self)-> list: ...

    @property
    def __d__(self)-> dict: ...



import os
from ..functions.builtins import HeEro
from ..functions.decorators import parse_self_x_item_value_or_set_by_instance_he_by_module_name
from .types.he import He


class System(He):
    def __init__(self):
        He.__init__(self, self)

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def components(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def configs(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def debugs(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def types(self): return

    def create_website(self, name:str=None): self.__i__.website_folder.copyto(self.components.folder(os.getcwd()).child_folder(name or 'website'))




class Condition(dict):
    def __init__(self, column:str, value: str | int | float, judge: str=None, logic:str=None, startwith:bool=None, endwith:bool=None):
        ...
    @property
    def column(self) -> str: ...
    @column.setter
    def column(self, value: str) -> None: ...
    @property
    def value(self) -> str | int | float: ...
    @judge.setter
    def value(self, _value: str | int | float) -> None: ...
    @property
    def judge(self): ...
    @judge.setter
    def judge(self, value: str) -> None: ...
    def set_judge_equal(self) -> None: ...
    def set_judge_un_equal(self) -> None: ...
    def set_judge_less_than(self) -> None: ...
    def set_judge_more_than(self) -> None: ...
    def set_judge_un_less_than(self) -> None: ...
    def set_judge_un_more_than(self) -> None: ...
    def set_judge_like(self) -> None: ...
    def set_judge_regexp(self) -> None: ...
    def set_judge_in_null(self) -> None: ...
    def set_judge_is_not_null(self) -> None: ...
    def set_judge_in(self) -> None: ...
    def set_judge_not_in(self) -> None: ...
    def set_judge_between_and(self) -> None: ...
    def set_judge_spatial(self) -> None: ...
    def set_judge_sounds_like(self) -> None: ...
    @property
    def logic(self) -> str: ...
    def set_logic(self, logic: str) -> None: ...
    def set_logic_and(self) -> None: ...
    def set_logic_or(self) -> None: ...
    @property
    def startwith(self) -> bool: ...
    @startwith.setter
    def startwith(self, value:bool=True) -> None: ...
    def set_startwith(self) -> None: ...
    @property
    def endwith(self) -> bool: ...
    @endwith.setter
    def endwith(self, value:bool=True) -> None: ...
    def set_endwith(self) -> None: ...

    def output(self, logic_requied:bool=False) -> str: ...
